// CirSim.java (c) 2009 by Paul Falstad

import java.io.InputStream;
import java.awt.*;
import java.awt.image.*;
import java.applet.Applet;
import java.util.Vector;
import java.io.File;
import java.util.Random;
import java.util.Arrays;
import java.lang.Math;
import java.net.URL;
import java.awt.event.*;
import java.io.FilterInputStream;
import java.io.ByteArrayOutputStream;
import java.util.StringTokenizer;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

public class CirSim extends Frame
  implements ComponentListener, ActionListener, AdjustmentListener,
  MouseMotionListener, MouseListener, ItemListener, KeyListener {
    
    Thread engine = null;

    Dimension winSize;
    Image dbimage;
    
    Random random;
    public static final int sourceRadius = 7;
    public static final double freqMult = 3.14159265*2*4;
    
    public String getAppletInfo() {
	return "AFilter by Paul Falstad";
    }

    static Container main;
    Label titleLabel;
    Button resetButton;
    Button dumpMatrixButton;
    MenuItem exportItem, importItem, exitItem, undoItem, redoItem,
	cutItem, copyItem, pasteItem, selectAllItem, optionsItem;
    Menu optionsMenu;
    Checkbox stoppedCheck;
    CheckboxMenuItem dotsCheckItem;
    CheckboxMenuItem voltsCheckItem;
    CheckboxMenuItem powerCheckItem;
    CheckboxMenuItem smallGridCheckItem;
    CheckboxMenuItem showValuesCheckItem;
    CheckboxMenuItem admittanceCheckItem;
    CheckboxMenuItem euroResistorCheckItem;
    CheckboxMenuItem printableCheckItem;
    CheckboxMenuItem conventionCheckItem;
    CheckboxMenuItem linearCheckItem;
    CheckboxMenuItem polesCheckItem;
    CheckboxMenuItem phaseCheckItem;
    Scrollbar speedBar;
    Scrollbar currentBar;
    Scrollbar scrollXBar;
    Scrollbar zoomBar;
    Label powerLabel, scrollXLabel;
    Scrollbar powerBar;
    PopupMenu elmMenu;
    MenuItem elmEditMenuItem;
    MenuItem elmCutMenuItem;
    MenuItem elmCopyMenuItem;
    MenuItem elmDeleteMenuItem;
    MenuItem elmScopeMenuItem;
    PopupMenu scopeMenu;
    PopupMenu transScopeMenu;
    PopupMenu mainMenu;
    CheckboxMenuItem scopeVMenuItem;
    CheckboxMenuItem scopeIMenuItem;
    CheckboxMenuItem scopeMaxMenuItem;
    CheckboxMenuItem scopeMinMenuItem;
    CheckboxMenuItem scopeFreqMenuItem;
    CheckboxMenuItem scopePowerMenuItem;
    CheckboxMenuItem scopeIbMenuItem;
    CheckboxMenuItem scopeIcMenuItem;
    CheckboxMenuItem scopeIeMenuItem;
    CheckboxMenuItem scopeVbeMenuItem;
    CheckboxMenuItem scopeVbcMenuItem;
    CheckboxMenuItem scopeVceMenuItem;
    CheckboxMenuItem scopeVIMenuItem;
    CheckboxMenuItem scopeXYMenuItem;
    CheckboxMenuItem scopeResistMenuItem;
    CheckboxMenuItem scopeVceIcMenuItem;
    MenuItem scopeSelectYMenuItem;
    Class addingClass;
    int mouseMode = MODE_SELECT;
    int tempMouseMode = MODE_SELECT;
    String mouseModeStr = "Select";
    static final double pi = 3.14159265358979323846;
    static final double log10 = 2.30258509299404568401;
    static final int MODE_ADD_ELM = 0;
    static final int MODE_DRAG_ALL = 1;
    static final int MODE_DRAG_ROW = 2;
    static final int MODE_DRAG_COLUMN = 3;
    static final int MODE_DRAG_SELECTED = 4;
    static final int MODE_DRAG_POST = 5;
    static final int MODE_SELECT = 6;
    static final int infoWidth = 120;
    int dragX, dragY, initDragX, initDragY;
    int selectedSource;
    Rectangle selectedArea;
    int gridSize, gridMask, gridRound;
    boolean dragging;
    boolean analyzeFlag;
    boolean dumpMatrix;
    boolean useBufferedImage;
    boolean isMac;
    String ctrlMetaKey;
    double t;
    int pause = 10;
    int scopeSelected = -1;
    int menuScope = -1;
    int hintType = -1, hintItem1, hintItem2;
    String stopMessage;
    double timeStep;
    double frequency, omega;
    double minFrequency = 10, maxFrequency = 25000;
    double freqLogRange, minLogFrequency;
    double selectedFreq, animateFreq;
    static final int HINT_LC = 1;
    static final int HINT_RC = 2;
    static final int HINT_3DB_C = 3;
    static final int HINT_TWINT = 4;
    static final int HINT_3DB_L = 5;
    Vector elmList;
    Vector setupList;
    CircuitElm dragElm, menuElm, mouseElm, stopElm;
    int mousePost = -1;
    CircuitElm plotXElm, plotYElm;
    int draggingPost;
    SwitchElm heldSwitchElm;
    Complex circuitMatrix[][], circuitRightSide[],
	origRightSide[], origMatrix[][];
    Complex phaseShift;
    RowInfo circuitRowInfo[];
    int circuitPermute[];
    boolean circuitNonLinear;
    int voltageSourceCount;
    int circuitMatrixSize, circuitMatrixFullSize;
    boolean circuitNeedsMap;
    public boolean useFrame;
    int scopeCount;
    Scope scopes[];
    int scopeColCount[];
    static EditDialog editDialog;
    static ImportDialog impDialog;
    Class dumpTypes[];
    static String muString = "u";
    static String ohmString = "ohm";
    String clipboard;
    Rectangle circuitArea, responseArea, circuitBbox, polesArea;
    int circuitBottom;
    Vector undoStack, redoStack;
    double response[], phaseResponse[], idealResponse[];
    double responseAdjust;
    int responseZero;
    Customizer customizer;

    int getrand(int x) {
	int q = random.nextInt();
	if (q < 0) q = -q;
	return q % x;
    }
    CircuitCanvas cv;
    AFilter applet;

    CirSim(AFilter a) {
	super("Analog Filter 0.91");
	applet = a;
	useFrame = false;
    }

    public void init() {
	String startCircuit = null;
	String startLabel = null;
	String euroResistor = null;
	String useFrameStr = null;
	boolean printable = false;
	boolean convention = true;

	CircuitElm.initClass(this);

	try {
	    String param = applet.getParameter("PAUSE");
	    if (param != null)
		pause = Integer.parseInt(param);
	    startCircuit = applet.getParameter("startCircuit");
	    startLabel   = applet.getParameter("startLabel");
	    euroResistor = applet.getParameter("euroResistors");
	    useFrameStr  = applet.getParameter("useFrame");
	    String x = applet.getParameter("whiteBackground");
	    if (x != null && x.equalsIgnoreCase("true"))
		printable = true;
	    x = applet.getParameter("conventionalCurrent");
	    if (x != null && x.equalsIgnoreCase("true"))
		convention = false;
	} catch (Exception e) { }
	
	if (startLabel == null)
	    startLabel = "High Pass Filter (RC)";
	if (startCircuit == null)
	    startCircuit = "filt-hipass.txt";
	boolean euro = (euroResistor != null && euroResistor.equalsIgnoreCase("true"));
	useFrame = (useFrameStr == null || !useFrameStr.equalsIgnoreCase("false"));
	if (useFrame)
	    main = this;
	else
	    main = applet;
	
	String os = System.getProperty("os.name");
	isMac = (os.indexOf("Mac ") == 0);
	ctrlMetaKey = (isMac) ? "\u2318" : "Ctrl";
	String jv = System.getProperty("java.class.version");
	double jvf = new Double(jv).doubleValue();
	if (jvf >= 48) {
	    muString = "\u03bc";
	    ohmString = "\u03a9";
	    useBufferedImage = true;
	}
	
	dumpTypes = new Class[300];
	// these characters are reserved
	dumpTypes[(int)'o'] = Scope.class;
	dumpTypes[(int)'h'] = Scope.class;
	dumpTypes[(int)'$'] = Scope.class;
	dumpTypes[(int)'%'] = Scope.class;
	dumpTypes[(int)'?'] = Scope.class;

	main.setLayout(new CircuitLayout());
	cv = new CircuitCanvas(this);
	cv.addComponentListener(this);
	cv.addMouseMotionListener(this);
	cv.addMouseListener(this);
	cv.addKeyListener(this);
	main.add(cv);

	mainMenu = new PopupMenu();
	MenuBar mb = null;
	if (useFrame)
	    mb = new MenuBar();
	Menu m = new Menu("File");
	if (useFrame)
	    mb.add(m);
	else
	    mainMenu.add(m);
	m.add(importItem = getMenuItem("Import"));
	m.add(exportItem = getMenuItem("Export"));
	m.addSeparator();
	m.add(exitItem   = getMenuItem("Exit"));

	m = new Menu("Edit");
	m.add(undoItem = getMenuItem("Undo"));
	undoItem.setShortcut(new MenuShortcut(KeyEvent.VK_Z));
	m.add(redoItem = getMenuItem("Redo"));
	redoItem.setShortcut(new MenuShortcut(KeyEvent.VK_Z, true));
	m.addSeparator();
	m.add(cutItem = getMenuItem("Cut"));
	cutItem.setShortcut(new MenuShortcut(KeyEvent.VK_X));
	m.add(copyItem = getMenuItem("Copy"));
	copyItem.setShortcut(new MenuShortcut(KeyEvent.VK_C));
	m.add(pasteItem = getMenuItem("Paste"));
	pasteItem.setShortcut(new MenuShortcut(KeyEvent.VK_V));
	pasteItem.setEnabled(false);
	m.add(selectAllItem = getMenuItem("Select All"));
	selectAllItem.setShortcut(new MenuShortcut(KeyEvent.VK_A));
	if (useFrame)
	    mb.add(m);
	else
	    mainMenu.add(m);

	m = new Menu("Scope");
	/*if (useFrame)
	    mb.add(m);
	else
	mainMenu.add(m);*/
	m.add(getMenuItem("Stack All", "stackAll"));
	m.add(getMenuItem("Unstack All", "unstackAll"));

	optionsMenu = m = new Menu("Options");
	if (useFrame)
	    mb.add(m);
	else
	    mainMenu.add(m);
	m.add(dotsCheckItem = getCheckItem("Show Current"));
	dotsCheckItem.setState(true);
	m.add(voltsCheckItem = getCheckItem("Show Voltage"));
	voltsCheckItem.setState(true);
	m.add(powerCheckItem = getCheckItem("Show Power"));
	m.add(showValuesCheckItem = getCheckItem("Show Values"));
	showValuesCheckItem.setState(true);
	m.add(admittanceCheckItem = getCheckItem("Show Admittance"));
	m.add(smallGridCheckItem = getCheckItem("Small Grid"));
	m.add(euroResistorCheckItem = getCheckItem("European Resistors"));
	euroResistorCheckItem.setState(euro);
	m.add(printableCheckItem = getCheckItem("White Background"));
	printableCheckItem.setState(printable);
	m.add(conventionCheckItem = getCheckItem("Conventional Current Motion"));
	conventionCheckItem.setState(convention);
 	m.add(linearCheckItem = getCheckItem("Linear Scale"));
 	m.add(polesCheckItem = getCheckItem("Show Poles & Zeroes"));
 	m.add(phaseCheckItem = getCheckItem("Show Phase"));
	m.add(optionsItem = getMenuItem("Other Options..."));
	
	Menu circuitsMenu = new Menu("Circuits");
	if (useFrame)
	    mb.add(circuitsMenu);
	else
	    mainMenu.add(circuitsMenu);

	mainMenu.add(getClassCheckItem("Add Wire", "WireElm"));
	mainMenu.add(getClassCheckItem("Add Resistor", "ResistorElm"));
	
	Menu passMenu = new Menu("Passive Components");
	mainMenu.add(passMenu);
	passMenu.add(getClassCheckItem("Add Capacitor", "CapacitorElm"));
	passMenu.add(getClassCheckItem("Add Inductor", "InductorElm"));
	passMenu.add(getClassCheckItem("Add Switch", "SwitchElm"));
	passMenu.add(getClassCheckItem("Add DPST Switch", "Switch2Elm"));
	passMenu.add(getClassCheckItem("Add Potentiometer", "PotElm"));
	passMenu.add(getClassCheckItem("Add Transmission Line", "TransLineElm"));
	
	Menu inputMenu = new Menu("Inputs/Outputs");
	mainMenu.add(inputMenu);
	inputMenu.add(getClassCheckItem("Add Ground", "GroundElm"));
 	inputMenu.add(getClassCheckItem("Add Input", "SweepElm"));
 	inputMenu.add(getClassCheckItem("Add Output", "OutputElm"));
	
	Menu activeMenu = new Menu("Active Components");
	mainMenu.add(activeMenu);
	activeMenu.add(getClassCheckItem("Add Op Amp (- on top)", "OpAmpElm"));
	activeMenu.add(getClassCheckItem("Add Op Amp (+ on top)",
				    "OpAmpSwapElm"));

	Menu otherMenu = new Menu("Other");
	mainMenu.add(otherMenu);
	otherMenu.add(getClassCheckItem("Add Text", "TextElm"));
 	getClassCheckItem("Add Scope Probe", "ProbeElm");
	otherMenu.add(getClassCheckItem("Add Box", "BoxElm"));
	otherMenu.add(getCheckItem("Drag All (Alt-drag)", "DragAll"));
	otherMenu.add(getCheckItem(
			  isMac ? "Drag Row (Alt-S-drag, S-right)" :
			  "Drag Row (S-right)",
			  "DragRow"));
	otherMenu.add(getCheckItem(
			  isMac ? "Drag Column (Alt-\u2318-drag, \u2318-right)" :
			  "Drag Column (C-right)",
			  "DragColumn"));
	otherMenu.add(getCheckItem("Drag Selected", "DragSelected"));
	otherMenu.add(getCheckItem("Drag Post (" + ctrlMetaKey + "-drag)",
				   "DragPost"));

	mainMenu.add(getCheckItem("Select/Drag Selected (space or Shift-drag)", "Select"));
	main.add(mainMenu);

	resetButton = new Button("Reset");
	resetButton.addActionListener(this);
	dumpMatrixButton = new Button("Dump Matrix");
	dumpMatrixButton.addActionListener(this);
	stoppedCheck = new Checkbox("Stopped");
	stoppedCheck.addItemListener(this);
	main.add(stoppedCheck);
	
	main.add(new Label("Simulation Speed", Label.CENTER));

	// was max of 140
	main.add(speedBar = new Scrollbar(Scrollbar.HORIZONTAL, 3, 1, -320,
					  260));
	speedBar.addAdjustmentListener(this);

	main.add(new Label("Current Speed", Label.CENTER));
	currentBar = new Scrollbar(Scrollbar.HORIZONTAL,
				   50, 1, 1, 100);
	currentBar.addAdjustmentListener(this);
	main.add(currentBar);

	main.add(new Label("Spectrum Width", Label.CENTER));
	zoomBar = new Scrollbar(Scrollbar.HORIZONTAL,
				   20, 1, 1, 100);
	zoomBar.addAdjustmentListener(this);
	main.add(zoomBar);

	main.add(powerLabel = new Label("Power Brightness", Label.CENTER));
	main.add(powerBar = new Scrollbar(Scrollbar.HORIZONTAL,
				    50, 1, 1, 100));
	powerBar.addAdjustmentListener(this);
	powerBar.disable();
	powerLabel.disable();

	main.add(scrollXLabel = new Label("Horiz Scroll", Label.CENTER));
	main.add(scrollXBar = new Scrollbar(Scrollbar.HORIZONTAL,
					    50, 1, 1, 100));
	scrollXBar.addAdjustmentListener(this);
	
	main.add(new Label("www.falstad.com"));

	if (useFrame)
	    main.add(new Label(""));
	Font f = new Font("SansSerif", 0, 10);
	Label l;
	l = new Label("Current Circuit:");
	l.setFont(f);
	titleLabel = new Label("Label");
	titleLabel.setFont(f);
	if (useFrame) {
	    main.add(l);
	    main.add(titleLabel);
	}

	setGrid();
	elmList = new Vector();
	setupList = new Vector();
	undoStack = new Vector();
	redoStack = new Vector();

	scopes = new Scope[20];
	scopeColCount = new int[20];
	scopeCount = 0;
	
	random = new Random();
	cv.setBackground(Color.black);
	cv.setForeground(Color.lightGray);
	
	elmMenu = new PopupMenu();
	elmMenu.add(elmEditMenuItem = getMenuItem("Edit"));
	elmScopeMenuItem = getMenuItem("View in Scope");
	elmMenu.add(elmCutMenuItem = getMenuItem("Cut"));
	elmMenu.add(elmCopyMenuItem = getMenuItem("Copy"));
	elmMenu.add(elmDeleteMenuItem = getMenuItem("Delete"));
	main.add(elmMenu);
	
	scopeMenu = buildScopeMenu(false);
	transScopeMenu = buildScopeMenu(true);
	setupResponse();

	getSetupList(circuitsMenu, false);
	if (useFrame)
	    setMenuBar(mb);
	if (stopMessage == null)
	    readSetupFile(startCircuit, startLabel);

	if (useFrame) {
	    Dimension screen = getToolkit().getScreenSize();
	    resize(860, 640);
	    handleResize();
	    Dimension x = getSize();
	    setLocation((screen.width  - x.width)/2,
			(screen.height - x.height)/2);
	    show();
	} else {
	    if (!powerCheckItem.getState()) {
		main.remove(powerBar);
		main.remove(powerLabel);
		main.validate();
	    }
	    hide();
	    handleResize();
	    applet.validate();
	}
	main.requestFocus();
    }

    boolean shown = false;
    
    public void triggerShow() {
	if (!shown)
	    show();
	shown = true;
    }
    
    PopupMenu buildScopeMenu(boolean t) {
	PopupMenu m = new PopupMenu();
	m.add(getMenuItem("Remove", "remove"));
	m.add(getMenuItem("Speed 2x", "speed2"));
	m.add(getMenuItem("Speed 1/2x", "speed1/2"));
	m.add(getMenuItem("Scale 2x", "scale"));
	m.add(getMenuItem("Max Scale", "maxscale"));
	m.add(getMenuItem("Stack", "stack"));
	m.add(getMenuItem("Unstack", "unstack"));
	m.add(getMenuItem("Reset", "reset"));
	if (t) {
	    m.add(scopeIbMenuItem = getCheckItem("Show Ib"));
	    m.add(scopeIcMenuItem = getCheckItem("Show Ic"));
	    m.add(scopeIeMenuItem = getCheckItem("Show Ie"));
	    m.add(scopeVbeMenuItem = getCheckItem("Show Vbe"));
	    m.add(scopeVbcMenuItem = getCheckItem("Show Vbc"));
	    m.add(scopeVceMenuItem = getCheckItem("Show Vce"));
	    m.add(scopeVceIcMenuItem = getCheckItem("Show Vce vs Ic"));
	} else {
	    m.add(scopeVMenuItem = getCheckItem("Show Voltage"));
	    m.add(scopeIMenuItem = getCheckItem("Show Current"));
	    m.add(scopePowerMenuItem = getCheckItem("Show Power Consumed"));
	    m.add(scopeMaxMenuItem = getCheckItem("Show Peak Value"));
	    m.add(scopeMinMenuItem = getCheckItem("Show Negative Peak Value"));
	    m.add(scopeFreqMenuItem = getCheckItem("Show Frequency"));
	    m.add(scopeVIMenuItem = getCheckItem("Show V vs I"));
	    m.add(scopeXYMenuItem = getCheckItem("Plot X/Y"));
	    m.add(scopeSelectYMenuItem = getMenuItem("Select Y", "selecty"));
	    m.add(scopeResistMenuItem = getCheckItem("Show Resistance"));
	}
	main.add(m);
	return m;
    }
    
    MenuItem getMenuItem(String s) {
	MenuItem mi = new MenuItem(s);
	mi.addActionListener(this);
	return mi;
    }

    MenuItem getMenuItem(String s, String ac) {
	MenuItem mi = new MenuItem(s);
	mi.setActionCommand(ac);
	mi.addActionListener(this);
	return mi;
    }

    CheckboxMenuItem getCheckItem(String s) {
	CheckboxMenuItem mi = new CheckboxMenuItem(s);
	mi.addItemListener(this);
	mi.setActionCommand("");
	return mi;
    }

    CheckboxMenuItem getClassCheckItem(String s, String t) {
	try {
	    Class c = Class.forName(t);
	    CircuitElm elm = constructElement(c, 0, 0);
	    register(c, elm);
	    int dt = 0;
	    if (elm.needsShortcut() && elm.getDumpClass() == c) {
		dt = elm.getDumpType();
		s += " (" + (char)dt + ")";
	    }
	    elm.delete();
	} catch (Exception ee) {
	    ee.printStackTrace();
	}
	return getCheckItem(s, t);
    }
    
    CheckboxMenuItem getCheckItem(String s, String t) {
	CheckboxMenuItem mi = new CheckboxMenuItem(s);
	mi.addItemListener(this);
	mi.setActionCommand(t);
	return mi;
    }

    void register(Class c, CircuitElm elm) {
	int t = elm.getDumpType();
	if (t == 0) {
	    System.out.println("no dump type: " + c);
	    return;
	}
	Class dclass = elm.getDumpClass();
	if (dumpTypes[t] == dclass)
	    return;
	if (dumpTypes[t] != null) {
	    System.out.println("dump type conflict: " + c + " " +
			       dumpTypes[t]);
	    return;
	}
	dumpTypes[t] = dclass;
    }
    
    void handleResize() {
        winSize = cv.getSize();
	if (winSize.width == 0)
	    return;
	dbimage = main.createImage(winSize.width, winSize.height);
	int h = winSize.height / 3;
	/*if (h < 128 && winSize.height > 300)
	  h = 128;*/
	int w = winSize.width;
	polesArea = null;
	if (polesCheckItem.getState()) {
	    w /= 3;
	    polesArea = new Rectangle(winSize.width-w, 0, w, winSize.height-h);
	}
	circuitArea = new Rectangle(0, 0, w, winSize.height-h);
	responseArea = new Rectangle(0, winSize.height-h, winSize.width*3/4, h);
	computeBbox();

	int minx = circuitBbox.x;
	int miny = circuitBbox.y;
	int maxx = minx + circuitBbox.width;
	int maxy = miny + circuitBbox.height;
	
	// center circuit; we don't use snapGrid() because that rounds
	int dx = gridMask & ((circuitArea.width -(maxx-minx))/2-minx);
	int dy = gridMask & ((circuitArea.height-(maxy-miny))/2-miny);
	if (dx+minx < 0)
	    dx = gridMask & (-minx);
	if (dy+miny < 0)
	    dy = gridMask & (-miny);
	moveAll(dx, dy);
	
	// after moving elements, need this to avoid singular matrix probs
	needAnalyze();
	
	circuitBottom = 0;
    }

    void computeBbox() {
	int i;
	int minx = 1000, maxx = 0, miny = 1000, maxy = 0;
	for (i = 0; i != elmList.size(); i++) {
	    CircuitElm ce = getElm(i);
	    // centered text causes problems when trying to center the circuit,
	    // so we special-case it here
	    if (!ce.isCenteredText()) {
		minx = min(ce.x, min(ce.x2, minx));
		maxx = max(ce.x, max(ce.x2, maxx));
	    }
	    miny = min(ce.y, min(ce.y2, miny));
	    maxy = max(ce.y, max(ce.y2, maxy));
	}
	circuitBbox = new Rectangle(minx, miny, maxx-minx, maxy-miny);
	int margin = 20;
	circuitBbox.grow(margin, margin);
	boolean fits = (maxx-minx < circuitArea.width);
	if (fits)
	    scrollXBar.setValues(0, circuitArea.width, 0, circuitArea.width);
	else
	    scrollXBar.setValues(-circuitBbox.x, circuitArea.width,
				 0, circuitBbox.width);
	boolean wasVisible = scrollXBar.isVisible();
	scrollXBar.setVisible(!fits);
	scrollXLabel.setVisible(!fits);
	if (wasVisible != !fits)
	    validate();
    }
    
    public boolean handleEvent(Event ev) {
        if (ev.id == Event.WINDOW_DESTROY) {
            if (applet != null) applet.destroyFrame();
            else dispose();
            return true;
        }
        return super.handleEvent(ev);
    }
    
    public void paint(Graphics g) {
	cv.repaint();
    }

    static final int resct = 6;
    long lastTime = 0, lastFrameTime, lastIterTime, secTime = 0;
    int frames = 0;
    int steps = 0;
    int framerate = 0, steprate = 0;

    public void updateCircuit(Graphics realg) {
	CircuitElm realMouseElm;
	if (winSize == null || winSize.width == 0)
	    return;
	if (analyzeFlag) {
	    analyzeCircuit();
	    analyzeFlag = false;
	}
 	calcResponse();
 	frequency = (animateFreq > 0) ? animateFreq : selectedFreq;
	if (editDialog != null && editDialog.elm instanceof CircuitElm)
	    mouseElm = (CircuitElm) (editDialog.elm);
	realMouseElm = mouseElm;
	if (mouseElm == null)
	    mouseElm = stopElm;
	setupScopes();
        Graphics g = null;
	g = dbimage.getGraphics();
	CircuitElm.selectColor = Color.cyan;
	if (printableCheckItem.getState()) {
	    CircuitElm.whiteColor = Color.black;
	    CircuitElm.lightGrayColor = Color.black;
	    g.setColor(Color.white);
	} else {
	    CircuitElm.whiteColor = Color.white;
	    CircuitElm.lightGrayColor = Color.lightGray;
	    g.setColor(Color.black);
	}
	g.fillRect(0, 0, winSize.width, winSize.height);
	if (!stoppedCheck.getState()) {
	    try {
		//runCircuit();
	    } catch (Exception e) {
		e.printStackTrace();
		analyzeFlag = true;
		cv.repaint();
		return;
	    }
	}
	if (!stoppedCheck.getState()) {
	    long sysTime = System.currentTimeMillis();
	    if (lastTime != 0) {
		int inc = (int) (sysTime-lastTime);
		double c = currentBar.getValue();
		c = java.lang.Math.exp(c/3.5-14.2);
		CircuitElm.currentMult = 1.7 * inc * c;
		if (!conventionCheckItem.getState())
		    CircuitElm.currentMult = -CircuitElm.currentMult;
	    }
	    if (sysTime-secTime >= 1000) {
		framerate = frames; steprate = steps;
		frames = 0; steps = 0;
		secTime = sysTime;
	    }
	    lastTime = sysTime;
	} else
	    lastTime = 0;
	CircuitElm.powerMult = Math.exp(powerBar.getValue()/4.762-7);
	
	int i;
	Font oldfont = g.getFont();
	// set omega for admittance display
	omega = 2*pi*selectedFreq;
	g.setColor(CircuitElm.whiteColor);
	for (i = 0; i != elmList.size(); i++) {
	    if (powerCheckItem.getState())
		g.setColor(Color.gray);
	    else if (admittanceCheckItem.getState())
		g.setColor(CircuitElm.whiteColor);
	    getElm(i).draw(g);
	}
	if (tempMouseMode == MODE_DRAG_ROW || tempMouseMode == MODE_DRAG_COLUMN ||
	    tempMouseMode == MODE_DRAG_POST || tempMouseMode == MODE_DRAG_SELECTED)
	    for (i = 0; i != elmList.size(); i++) {
		CircuitElm ce = getElm(i);
		ce.drawPost(g, ce.x , ce.y );
		ce.drawPost(g, ce.x2, ce.y2);
	    }
	/*if (mouseElm != null) {
	    g.setFont(oldfont);
	    g.drawString("+", mouseElm.x+10, mouseElm.y);
	    }*/
	if (dragElm != null &&
	      (dragElm.x != dragElm.x2 || dragElm.y != dragElm.y2))
	    dragElm.draw(g);
	g.setFont(oldfont);
	int ct = scopeCount;
	if (stopMessage != null)
	    ct = 0;
	/*
	for (i = 0; i != ct; i++)
	    scopes[i].draw(g);
	*/
	g.setColor(CircuitElm.whiteColor);
	double respValue = 0, phaseValue = 0;
	if (responseArea != null) {
	    int bottom = responseArea.y + responseArea.height;
	    g.setColor(Color.darkGray);
	    g.fillRect(responseArea.x, responseArea.y,
		       responseArea.width, responseArea.height);
	    double ym = .069;
	    for (i = 0; ; i += 2) {
		double q = ym*i;
		if (q > 1)
		    break;
		g.setColor(i == 2*responseZero ? Color.gray : Color.black);
		int y = responseArea.y + (int) (q*responseArea.height);
		g.drawLine(responseArea.x, y, responseArea.width, y);
	    }
	    double fq = 1;
	    double fqstep = 1;
	    while (true) {
		if (fq > maxFrequency)
		    break;
		if (fq >= minFrequency) {
		    int x = (int) (frequencyToLinear(fq)*responseArea.width);
		    g.drawLine(x, responseArea.y, x, bottom);
		}
		fq += fqstep;
		if (fq == fqstep * 10)
		    fqstep *= 10;
	    }
	    g.setColor(Color.red);
	    if (idealResponse != null) {
		int ox = -1, oy = -1;
		for (i = 0; i != responseArea.width; i++) {
		    double w = 0;
		    double bw = idealResponse[i*response.length/responseArea.width];
		    double val = -ym*Math.log(bw*bw*responseAdjust)/log10;
		    int x = i+responseArea.x;
		    if (val > 1) {
			if (ox != -1)
			    g.drawLine(ox, oy, ox, bottom);
			ox = -1;
		    } else {
			int y = responseArea.y + (int) (responseArea.height*val);
			if (ox != -1)
			    g.drawLine(ox, oy, x, y);
			else if (x > responseArea.x)
			    g.drawLine(x, bottom, x, y);
			ox = x;
			oy = y;
		    }
		}
	    }
	    int ox = -1, oy = -1;
	    g.setColor(Color.white);
	    for (i = 0; i != responseArea.width; i++) {
		double w = 0;
		double bw = response[i*response.length/responseArea.width];
		double val = -ym*Math.log(bw*bw*responseAdjust)/log10;
		/*if (linRespCheckItem.getState())
		  val = 1-cc.mag;*/
		int x = i+responseArea.x;
		if (val > 1) {
		    if (ox != -1)
			g.drawLine(ox, oy, ox, bottom);
		    ox = -1;
		} else {
		    int y = responseArea.y + (int) (responseArea.height*val);
		    if (ox != -1)
			g.drawLine(ox, oy, x, y);
		    else if (x > responseArea.x)
			g.drawLine(x, bottom, x, y);
		    ox = x;
		    oy = y;
		}
	    }
	    if (phaseCheckItem.getState()) {
		int phaseHeight = responseArea.height / 2;
		int ty = winSize.height-phaseHeight;
		int my = winSize.height-phaseHeight/2;
		g.setColor(Color.darkGray);
		g.fillRect(responseArea.x, ty,
			   responseArea.width, phaseHeight);
		g.setColor(Color.black);
		g.drawLine(responseArea.x, ty,
			   responseArea.width+responseArea.x, ty);
		g.drawLine(responseArea.x, my,
			   responseArea.width+responseArea.x, my);
		g.setColor(Color.white);
		ox = oy = -1;
		for (i = 0; i != responseArea.width; i++) {
		    double w = 0;
		    double val = -(phaseResponse[i*response.length/
						 responseArea.width]/(2*pi));
		    /*if (linRespCheckItem.getState())
		      val = 1-cc.mag;*/
		    int x = i+responseArea.x;
		    if (val > 1) {
			if (ox != -1)
			    g.drawLine(ox, oy, ox, bottom);
			ox = -1;
		    } else {
			int y = my + (int) ((phaseHeight-1)*val);
			if (ox != -1)
			    g.drawLine(ox, oy, x, y);
			else if (x > responseArea.x)
			    g.drawLine(x, bottom, x, y);
			ox = x;
			oy = y;
		    }
		}
	    }
	    g.setColor(Color.yellow);
	    if (selectedFreq > 0) {
		int x = (int)
		    (frequencyToLinear(selectedFreq)*responseArea.width);
		g.drawLine(x, responseArea.y, x, bottom);
		double bw = response[x*response.length/responseArea.width];
		respValue = 10*Math.log(bw*bw)/log10;
		phaseValue = phaseResponse[x*response.length/
					   responseArea.width];
		omega = 2*pi*selectedFreq;
	    }
	    
	    if (specStep != 0) {
		FontMetrics fm = g.getFontMetrics();
		g.setColor(Color.black);
		String cs = "Calculating...";
		g.fillRect(0, winSize.height-30, 20+fm.stringWidth(cs), 30);
		g.setColor(Color.white);
		g.drawString(cs, 10, winSize.height-10);
	    }
	}
	if (polesArea != null)
	    drawPoles(g);
	g.setColor(Color.white);
	if (stopMessage != null) {
	    g.drawString(stopMessage, 10, circuitArea.height);
	} else {
	    if (!stoppedCheck.getState() && animateFreq > 0)
		animateCircuit();
	    if (circuitBottom == 0)
		calcCircuitBottom();
	    String info[] = new String[10];
	    if (mouseElm != null) {
		if (mousePost == -1)
		    mouseElm.getInfo(info);
		else
		    info[0] = "V = " +
			CircuitElm.getUnitText(mouseElm.getPostVoltage(mousePost), "V");
		/* //shownodes
		for (i = 0; i != mouseElm.getPostCount(); i++)
		    info[0] += " " + mouseElm.nodes[i];
		if (mouseElm.getVoltageSourceCount() > 0)
		    info[0] += ";" + (mouseElm.getVoltageSource()+nodeList.size());
		*/
		
	    } else {
		CircuitElm.showFormat.setMinimumFractionDigits(2);
		info[0] = "t = " + CircuitElm.getUnitText(t, "s");
 		if (animateFreq > 0)
 		    info[1] = "f = " +
 			CircuitElm.getUnitText(animateFreq, "Hz");
		CircuitElm.showFormat.setMinimumFractionDigits(0);
	    }
	    for (i = 0; info[i] != null; i++)
		;
	    if (hintType != -1) {
		String s = getHint();
		if (s == null)
		    hintType = -1;
		else
 		    info[i++] = s;
	    }
 	    if (selectedFreq != 0) {
 		info[i++] = CircuitElm.getUnitText(selectedFreq, "Hz") +
 		    ": " + CircuitElm.showFormat.format(respValue) + " dB";
		if (phaseCheckItem.getState())
		    info[i++] = "Phase: " +
			CircuitElm.showFormat.format(phaseValue*180/pi)
			+ "\u00b0";
	    }
 	    /*if (ct != 0)
 	      x = scopes[ct-1].rightEdge() + 20;*/
 	    //x = max(0, winSize.width*2/3);
 	    if (customizer != null && mouseElm == null)
 		customizer.getInfo(info, i);
	    
	    // count lines of data
	    for (i = 0; info[i] != null; i++)
		;
	    
	    // find where to show data; below circuit, not too high unless we need it
	    int ybase = winSize.height-15*i-5;
	    ybase = min(ybase, circuitArea.height);
	    ybase = max(ybase, circuitBottom);
	    int x = responseArea.width;
	    for (i = 0; info[i] != null; i++)
		g.drawString(info[i], x,
			     ybase+15*(i+1));
	}
	if (selectedArea != null) {
	    g.setColor(CircuitElm.selectColor);
	    g.drawRect(selectedArea.x, selectedArea.y, selectedArea.width, selectedArea.height);
	}
	mouseElm = realMouseElm;
	frames++;
	/*
	g.setColor(Color.white);
	g.drawString("Framerate: " + framerate, 10, 10);
	g.drawString("Steprate: " + steprate,  10, 30);
	g.drawString("Steprate/iter: " + (steprate/getIterCount()),  10, 50);
	g.drawString("iterc: " + (getIterCount()),  10, 70);
	*/

	realg.drawImage(dbimage, 0, 0, this);
	if (!stoppedCheck.getState() && circuitMatrix != null &&
	    animateFreq > 0) {
	    // Limit to 50 fps (thanks to Jrgen Kltzer for this)
	    long delay = 1000/50 - (System.currentTimeMillis() - lastFrameTime);
	    //realg.drawString("delay: " + delay,  10, 90);
	    if (delay > 0) {
		try {
		    Thread.sleep(delay);
		} catch (InterruptedException e) {
		}
	    }
	    
	    cv.repaint(0);
	}
	lastFrameTime = lastTime;
    }

    void drawPoles(Graphics g) {
	Complex poles[] = null;
	if (customizer != null)
	    poles = customizer.getPoles();
	if (poles == null)
	    poles = getHintPoles();
	//System.out.println(hintType + " " + customizer + " " + poles);
	if (poles == null) {
	    g.setColor(Color.black);
	    g.fillRect(polesArea.x, polesArea.y,
		       polesArea.width, polesArea.height);
	    return;
	}
	g.setColor(Color.darkGray);
	g.fillRect(polesArea.x, polesArea.y,
		   polesArea.width, polesArea.height);
	int rx = polesArea.x+polesArea.width-10;
	int my = polesArea.y + polesArea.height/2;
	int i;
	double mx = 1;
	if (customizer != null && customizer instanceof FilterCustomizer) {
	    double f = ((FilterCustomizer) customizer).getFrequency();
	    mx = 2*Math.PI*f;
	}
	for (i = 0; i != poles.length; i++) {
	    Complex c = poles[i];
	    if (Math.abs(c.re) > mx)
		mx = Math.abs(c.re);
	    if (Math.abs(c.im) > mx)
		mx = Math.abs(c.im);
	}
	int m0 = polesArea.height/2-10;
	if (m0 > polesArea.width-10)
	    m0 = polesArea.width-10;
	double scale = 1;
	while (scale*mx > m0)
	    scale /= 2;
	double grid = 1e-8;
	while (grid*scale*10 < m0)
	    grid *= 10;
	g.setColor(Color.black);
	for (i = 0; i != 20; i++) {
	    int cx = (int) (rx-grid*i*scale);
	    if (cx > polesArea.x)
		g.drawLine(cx, polesArea.y, cx, polesArea.y+polesArea.height);
	    int cy = (int) (my-grid*i*scale);
	    g.drawLine(polesArea.x, cy, polesArea.x+polesArea.width, cy);
	    cy = (int) (my+grid*i*scale);
	    if (cy < polesArea.y+polesArea.height)
		g.drawLine(polesArea.x, cy, polesArea.x+polesArea.width, cy);
	}
	if (customizer != null && customizer instanceof FilterCustomizer) {
	    double f = ((FilterCustomizer) customizer).getFrequency();
	    int or = (int) (2*Math.PI*f*scale);
	    g.drawOval(rx-or, my-or, or*2, or*2);
	}
	g.setColor(Color.white);
	g.drawLine(polesArea.x, my,
		   polesArea.x+polesArea.width, my);
	g.drawLine(rx, polesArea.y, rx, polesArea.y+polesArea.height-1);
	int selnum = -1;
	if (customizer instanceof LowActiveFilter ||
	    customizer instanceof HighActiveFilter) {
	    for (i = 0; i != countElm(BoxElm.class); i++)
		if (mouseElm == getElm(i, BoxElm.class))
		    selnum = i;
	}
	for (i = 0; i != poles.length; i++) {
	    Complex c = poles[i];
	    g.setColor((i == selnum || i == poles.length-1-selnum) ?
		       CircuitElm.selectColor : Color.white);
	    int cx = (int) (rx+c.re*scale);
	    int cy = (int) (my-c.im*scale);
	    g.drawLine(cx-2, cy-2, cx+2, cy+2);
	    g.drawLine(cx+2, cy-2, cx-2, cy+2);
	}
	Complex zeros[] = customizer.getZeros();
	if (zeros != null) {
	    for (i = 0; i != zeros.length; i++) {
		Complex c = zeros[i];
		int cx = (int) (rx+c.re*scale);
		int cy = (int) (my-c.im*scale);
		g.drawOval(cx-3, cy-3, 7, 7);
	    }
	}
	if (selectedFreq > 0) {
	    g.setColor(Color.yellow);
	    int cy = (int) (my-scale*2*pi*selectedFreq);
	    g.drawLine(polesArea.x, cy, rx, cy);
	}
    }
    
    void setupScopes() {
	int i;
	
	// check scopes to make sure the elements still exist, and remove
	// unused scopes/columns
	int pos = -1;
	for (i = 0; i < scopeCount; i++) {
	    if (locateElm(scopes[i].elm) < 0)
		scopes[i].setElm(null);
	    if (scopes[i].elm == null) {
		int j;
		for (j = i; j != scopeCount; j++)
		    scopes[j] = scopes[j+1];
		scopeCount--;
		i--;
		continue;
	    }
	    if (scopes[i].position > pos+1)
		scopes[i].position = pos+1;
	    pos = scopes[i].position;
	}
	while (scopeCount > 0 && scopes[scopeCount-1].elm == null)
	    scopeCount--;
	int h = winSize.height - circuitArea.height;
	pos = 0;
	for (i = 0; i != scopeCount; i++)
	    scopeColCount[i] = 0;
	for (i = 0; i != scopeCount; i++) {
	    pos = max(scopes[i].position, pos);
	    scopeColCount[scopes[i].position]++;
	}
	int colct = pos+1;
	int iw = infoWidth;
	if (colct <= 2)
	    iw = iw*3/2;
	int w = (winSize.width-iw) / colct;
	int marg = 10;
	if (w < marg*2)
	    w = marg*2;
	pos = -1;
	int colh = 0;
	int row = 0;
	int speed = 0;
	for (i = 0; i != scopeCount; i++) {
	    Scope s = scopes[i];
	    if (s.position > pos) {
		pos = s.position;
		colh = h / scopeColCount[pos];
		row = 0;
		speed = s.speed;
	    }
	    if (s.speed != speed) {
		s.speed = speed;
		s.resetGraph();
	    }
	    Rectangle r = new Rectangle(pos*w, winSize.height-h+colh*row,
					w-marg, colh);
	    row++;
	    if (!r.equals(s.rect))
		s.setRect(r);
	}
    }
    
    String getHint() {
	CircuitElm c1 = getElm(hintItem1);
	CircuitElm c2 = getElm(hintItem2);
	if (c1 == null || c2 == null)
	    return null;
	if (hintType == HINT_LC) {
	    if (!(c1 instanceof InductorElm))
		return null;
	    if (!(c2 instanceof CapacitorElm))
		return null;
	    InductorElm ie = (InductorElm) c1;
	    CapacitorElm ce = (CapacitorElm) c2;
	    return "res.f = " + CircuitElm.getUnitText(1/(2*pi*Math.sqrt(ie.inductance*
						    ce.capacitance)), "Hz");
	}
	if (hintType == HINT_RC) {
	    if (!(c1 instanceof ResistorElm))
		return null;
	    if (!(c2 instanceof CapacitorElm))
		return null;
	    ResistorElm re = (ResistorElm) c1;
	    CapacitorElm ce = (CapacitorElm) c2;
	    return "RC = " + CircuitElm.getUnitText(re.resistance*ce.capacitance,
					 "s");
	}
	if (hintType == HINT_3DB_C) {
	    if (!(c1 instanceof ResistorElm))
		return null;
	    if (!(c2 instanceof CapacitorElm))
		return null;
	    ResistorElm re = (ResistorElm) c1;
	    CapacitorElm ce = (CapacitorElm) c2;
	    return "f.3db = " +
		CircuitElm.getUnitText(1/(2*pi*re.resistance*ce.capacitance), "Hz");
	}
	if (hintType == HINT_3DB_L) {
	    if (!(c1 instanceof ResistorElm))
		return null;
	    if (!(c2 instanceof InductorElm))
		return null;
	    ResistorElm re = (ResistorElm) c1;
	    InductorElm ie = (InductorElm) c2;
	    return "f.3db = " +
		CircuitElm.getUnitText(re.resistance/(2*pi*ie.inductance), "Hz");
	}
	if (hintType == HINT_TWINT) {
	    if (!(c1 instanceof ResistorElm))
		return null;
	    if (!(c2 instanceof CapacitorElm))
		return null;
	    ResistorElm re = (ResistorElm) c1;
	    CapacitorElm ce = (CapacitorElm) c2;
	    return "fc = " +
		CircuitElm.getUnitText(1/(2*pi*re.resistance*ce.capacitance), "Hz");
	}
	return null;
    }

    Complex [] getHintPoles() {
	CircuitElm c1 = getElm(hintItem1);
	CircuitElm c2 = getElm(hintItem2);
	if (c1 == null || c2 == null)
	    return null;
	/*
	if (hintType == HINT_LC) {
	    if (!(c1 instanceof InductorElm))
		return null;
	    if (!(c2 instanceof CapacitorElm))
		return null;
	    InductorElm ie = (InductorElm) c1;
	    CapacitorElm ce = (CapacitorElm) c2;
	    return "res.f = " + CircuitElm.getUnitText(1/(2*pi*Math.sqrt(ie.inductance*
						    ce.capacitance)), "Hz");
	}
	if (hintType == HINT_RC) {
	    if (!(c1 instanceof ResistorElm))
		return null;
	    if (!(c2 instanceof CapacitorElm))
		return null;
	    ResistorElm re = (ResistorElm) c1;
	    CapacitorElm ce = (CapacitorElm) c2;
	    return "RC = " + CircuitElm.getUnitText(re.resistance*ce.capacitance,
					 "s");
	}
	*/
	if (hintType == HINT_3DB_C) {
	    if (!(c1 instanceof ResistorElm))
		return null;
	    if (!(c2 instanceof CapacitorElm))
		return null;
	    ResistorElm re = (ResistorElm) c1;
	    CapacitorElm ce = (CapacitorElm) c2;
	    return new Complex[] { new Complex(-1/(re.resistance*ce.capacitance), 0) };
	}
	if (hintType == HINT_3DB_L) {
	    if (!(c1 instanceof ResistorElm))
		return null;
	    if (!(c2 instanceof InductorElm))
		return null;
	    ResistorElm re = (ResistorElm) c1;
	    InductorElm ie = (InductorElm) c2;
	    return new Complex[] { new Complex(-re.resistance/ie.inductance, 0) };
	}
	/*
	if (hintType == HINT_TWINT) {
	    if (!(c1 instanceof ResistorElm))
		return null;
	    if (!(c2 instanceof CapacitorElm))
		return null;
	    ResistorElm re = (ResistorElm) c1;
	    CapacitorElm ce = (CapacitorElm) c2;
	    return "fc = " +
		CircuitElm.getUnitText(1/(2*pi*re.resistance*ce.capacitance), "Hz");
	}
	*/
	return null;
    }

    public void toggleSwitch(int n) {
	int i;
	for (i = 0; i != elmList.size(); i++) {
	    CircuitElm ce = getElm(i);
	    if (ce instanceof SwitchElm) {
		n--;
		if (n == 0) {
		    ((SwitchElm) ce).toggle();
		    analyzeFlag = true;
		    cv.repaint();
		    return;
		}
	    }
	}
    }
    
    void editPerformed(Editable elm) {
	if (customizer != null) {
	    if (!customizer.editPerformed(elm))
		releaseCustomizer();
	}
	needAnalyze();
    }
    
    void needAnalyze() {
	analyzeFlag = true;
	cv.repaint();
    }
    
    Vector nodeList;
    CircuitElm voltageSources[];

    public CircuitNode getCircuitNode(int n) {
	if (n >= nodeList.size())
	    return null;
	return (CircuitNode) nodeList.elementAt(n);
    }

    public CircuitElm getElm(int n) {
	if (n >= elmList.size())
	    return null;
	return (CircuitElm) elmList.elementAt(n);
    }

    public int countElm(Class c) {
	int i;
	int count = 0;
	for (i = 0; i != elmList.size(); i++) {
	    if (c.isInstance(elmList.elementAt(i)))
		count++;
	}
	return count;
    }
    
    public CircuitElm getElm(int n, Class c) {
	int i;
	for (i = 0; i != elmList.size(); i++) {
	    if (c.isInstance(elmList.elementAt(i)) && n-- == 0)
		return (CircuitElm) elmList.elementAt(i);
	}
	return null;
    }
    
     void analyzeCircuit() {
	specStep = 0;
	calcCircuitBottom();
	if (elmList.isEmpty())
	    return;
	stopMessage = null;
	stopElm = null;
	int i, j;
	int vscount = 0;
	nodeList = new Vector();
	boolean gotGround = false;
	boolean gotRail = false;
	CircuitElm volt = null;

	//System.out.println("ac1");
	// look for voltage or ground element
	for (i = 0; i != elmList.size(); i++) {
	    CircuitElm ce = getElm(i);
	    if (ce instanceof GroundElm) {
		gotGround = true;
		break;
	    }
	    if (ce instanceof RailElm)
		gotRail = true;
	    if (volt == null && ce instanceof VoltageElm)
		volt = ce;
	}

	// if no ground, and no rails, then the voltage elm's first terminal
	// is ground
	if (!gotGround && volt != null && !gotRail) {
	    CircuitNode cn = new CircuitNode();
	    Point pt = volt.getPost(0);
	    cn.x = pt.x;
	    cn.y = pt.y;
	    nodeList.addElement(cn);
	} else {
	    // otherwise allocate extra node for ground
	    CircuitNode cn = new CircuitNode();
	    cn.x = cn.y = -1;
	    nodeList.addElement(cn);
	}
	//System.out.println("ac2");

	// allocate nodes and voltage sources
	for (i = 0; i != elmList.size(); i++) {
	    CircuitElm ce = getElm(i);
	    int inodes = ce.getInternalNodeCount();
	    int ivs = ce.getVoltageSourceCount();
	    int posts = ce.getPostCount();
	    
	    // allocate a node for each post and match posts to nodes
	    for (j = 0; j != posts; j++) {
		Point pt = ce.getPost(j);
		int k;
		for (k = 0; k != nodeList.size(); k++) {
		    CircuitNode cn = getCircuitNode(k);
		    if (pt.x == cn.x && pt.y == cn.y)
			break;
		}
		if (k == nodeList.size()) {
		    CircuitNode cn = new CircuitNode();
		    cn.x = pt.x;
		    cn.y = pt.y;
		    CircuitNodeLink cnl = new CircuitNodeLink();
		    cnl.num = j;
		    cnl.elm = ce;
		    cn.links.addElement(cnl);
		    ce.setNode(j, nodeList.size());
		    nodeList.addElement(cn);
		} else {
		    CircuitNodeLink cnl = new CircuitNodeLink();
		    cnl.num = j;
		    cnl.elm = ce;
		    getCircuitNode(k).links.addElement(cnl);
		    ce.setNode(j, k);
		    // if it's the ground node, make sure the node voltage is 0,
		    // cause it may not get set later
		    if (k == 0)
			ce.setNodeVoltage(j, 0, 0);
		}
	    }
	    for (j = 0; j != inodes; j++) {
		CircuitNode cn = new CircuitNode();
		cn.x = cn.y = -1;
		cn.internal = true;
		CircuitNodeLink cnl = new CircuitNodeLink();
		cnl.num = j+posts;
		cnl.elm = ce;
		cn.links.addElement(cnl);
		ce.setNode(cnl.num, nodeList.size());
		nodeList.addElement(cn);
	    }
	    vscount += ivs;
	}
	voltageSources = new CircuitElm[vscount];
	vscount = 0;
	circuitNonLinear = false;
	//System.out.println("ac3");

	// set voltage sources
	for (i = 0; i != elmList.size(); i++) {
	    CircuitElm ce = getElm(i);
	    int ivs = ce.getVoltageSourceCount();
	    for (j = 0; j != ivs; j++) {
		voltageSources[vscount] = ce;
		ce.setVoltageSource(j, vscount++);
	    }
	}
	voltageSourceCount = vscount;

	int matrixSize = nodeList.size()-1 + vscount;
	circuitMatrix = new Complex[matrixSize][matrixSize];
	circuitRightSide = new Complex[matrixSize];
	origMatrix = new Complex[matrixSize][matrixSize];
	origRightSide = new Complex[matrixSize];
	circuitMatrixSize = circuitMatrixFullSize = matrixSize;
	circuitRowInfo = new RowInfo[matrixSize];
	circuitPermute = new int[matrixSize];
	phaseShift = new Complex();
	int vs = 0;
	for (i = 0; i != matrixSize; i++)
	    circuitRowInfo[i] = new RowInfo();
	for (i = 0; i != matrixSize; i++)
	    circuitRightSide[i] = new Complex();
	for (i = 0; i != matrixSize; i++)
	    for (j = 0; j != matrixSize; j++)
		circuitMatrix[i][j] = new Complex();
	circuitNeedsMap = false;
	needsClosure = true;
	response = new double[responseArea == null ? 1024 : responseArea.width];
	phaseResponse = new double[response.length];
	specStep = 1;
	while (specStep < response.length)
	    specStep *= 2;
	specIndex = specStep/2;
	calcResponse();
	idealResponse = null;
	if (customizer != null)
	    idealResponse = customizer.getIdealResponse(response.length);
    }

    int specIndex, specStep;
    boolean needsClosure = true;
    
    void calcResponse() {
	int i, j, fi;
	int matrixSize = circuitMatrixSize;
	long tm = System.currentTimeMillis() + 100;
	if (specStep == 0)
	    return;
	while (specStep != 0 && System.currentTimeMillis() <= tm) {
	    fi = specIndex-1;
	    frequency = linearToFrequency(fi/(double) response.length);
	    solveCircuit();
	    
	    for (i = 0; i != elmList.size(); i++) {
		CircuitElm ce = getElm(i);
		if (ce instanceof OutputElm) {
		    int n = ce.getNode(0);
		    response[fi] = circuitRightSide[n-1].mag;
		    phaseResponse[fi] = circuitRightSide[n-1].phase;
		}
	    }

	    // interpolate
	    int prev = specIndex-specStep/2-1;
	    if (prev >= 0) {
		for (i = prev+1; i < fi; i++) {
		    double w = (i-prev)/(fi-(double)prev);
		    response[i] = response[prev]*(1-w) + response[fi]*w;
		    phaseResponse[i] = phaseResponse[prev]*(1-w) +
			phaseResponse[fi]*w;
		}
	    }
	    int next = specIndex+specStep/2-1;
	    if (next < response.length) {
		for (i = fi+1; i < next; i++) {
		    double w = (next-i)/(next-(double)fi);
		    response[i] = response[next]*(1-w) + response[fi]*w;
		    phaseResponse[i] =
			phaseResponse[next]*(1-w) + phaseResponse[fi]*w;
		}
	    }
	    
	    // next step
	    specIndex += specStep;
	    if (specIndex > response.length) {
		specStep /= 2;
		specIndex = specStep/2;
		if (specStep == 1)
		    specStep = 0;
	    }
	}
	double maxResponse = 0;
	for (i = 0; i != response.length; i++)
	    if (response[i] > maxResponse)
		maxResponse = response[i];
	responseAdjust = 1;
	responseZero = 0;
	while (maxResponse*maxResponse*responseAdjust > 1) {
	    responseAdjust /= 100;
	    responseZero++;
	}
	if (specStep != 0)
	    cv.repaint();
    }

    void solveCircuit() {
	int i, j;
	int matrixSize = circuitMatrixSize;
	for (i = 0; i != matrixSize; i++)
	    for (j = 0; j != matrixSize; j++)
		circuitMatrix[i][j].set(0, 0);
	for (i = 0; i != matrixSize; i++)
	    circuitRightSide[i].set(0, 0);
	omega = 2*pi*frequency;
	
	// stamp linear circuit elements
	for (i = 0; i != elmList.size(); i++) {
	    CircuitElm ce = getElm(i);
	    ce.stamp();
	}
	
	if (needsClosure) {
	    // determine nodes that are unconnected
	    boolean closure[] = new boolean[nodeList.size()];
	    boolean changed = true;
	    boolean neededClosure = false;
	    closure[0] = true;
	    while (changed) {
		changed = false;
		for (i = 0; i != elmList.size(); i++) {
		    CircuitElm ce = getElm(i);
		    // loop through all ce's nodes to see if they are connected
		    // to other nodes not in closure
		    for (j = 0; j < ce.getPostCount(); j++) {
			if (!closure[ce.getNode(j)]) {
			    if (ce.hasGroundConnection(j))
				closure[ce.getNode(j)] = changed = true;
			    continue;
			}
			int k;
			for (k = 0; k != ce.getPostCount(); k++) {
			    if (j == k)
				continue;
			    int kn = ce.getNode(k);
			    if (ce.getConnection(j, k) && !closure[kn]) {
				closure[kn] = true;
				changed = true;
			    }
			}
		    }
		}
		if (changed)
		    continue;
		
		// connect unconnected nodes
		for (i = 0; i != nodeList.size(); i++)
		    if (!closure[i] && !getCircuitNode(i).internal) {
			System.out.println("node " + i + " unconnected");
			stampResistor(0, i, 1e8);
			neededClosure = true;
			closure[i] = true;
			changed = true;
			break;
		    }
	    }
	    needsClosure = neededClosure;
	}
	
	circuitMatrixSize = matrixSize;
	if (false) {
	    for (j = 0; j != circuitMatrixSize; j++) {
		for (i = 0; i != circuitMatrixSize; i++)
		    System.out.print(circuitMatrix[j][i].asString() + ",");
		System.out.print("  " + circuitRightSide[j].asString() + "\n");
	    }
	    System.out.print("\n");
	}
	if (!lu_factor(circuitMatrix, circuitMatrixSize, circuitPermute)) {
	    stop("Singular matrix!", null);
	    return;
	}
	
	lu_solve(circuitMatrix, circuitMatrixSize, circuitPermute,
		 circuitRightSide);
    }

    void animateCircuit() {
	frequency = animateFreq;
	solveCircuit();
	double steprate = 160*getIterCount();
	long tm = System.currentTimeMillis();
	double tadd = (tm-lastIterTime)*steprate*timeStep/1000;
	if (lastIterTime == 0)
	    tadd = 0;
	t += tadd;
	double phase = frequency*2*pi*t;
	phaseShift.set(1, 0);
	phaseShift.rotate(phase);
	int i, j, k;
	for (j = 0; j != circuitMatrixFullSize; j++) {
	    circuitRightSide[j].rotate(phase);
	    double vr = circuitRightSide[j].re * CircuitElm.voltageRange;
	    double vi = circuitRightSide[j].im * CircuitElm.voltageRange;
	    if (j < nodeList.size()-1) {
		CircuitNode cn = getCircuitNode(j+1);
		for (k = 0; k != cn.links.size(); k++) {
		    CircuitNodeLink cnl = (CircuitNodeLink)
			cn.links.elementAt(k);
		    cnl.elm.setNodeVoltage(cnl.num, vr, vi);
		}
	    } else {
		int ji = j-(nodeList.size()-1);
		//System.out.println("setting vsrc " + ji + " to " + res);
		voltageSources[ji].setCurrent(ji, vr, vi);
	    }
	}
	lastIterTime = tm;
    }
    
    Complex [][] build_matrix(int sz) {
	int i,j;
	Complex m[][] = new Complex[sz][sz];
	for (i = 0; i != sz; i++)
	    for (j = 0; j != sz; j++)
		m[i][j] = new Complex();
	return m;
    }

    Complex [] build_vector(int sz) {
	int i;
	Complex m[] = new Complex[sz];
	for (i = 0; i != sz; i++)
	    m[i] = new Complex();
	return m;
    }
    
    void calcCircuitBottom() {
	int i;
	circuitBottom = 0;
	for (i = 0; i != elmList.size(); i++) {
	    Rectangle rect = getElm(i).boundingBox;
	    int bottom = rect.height + rect.y;
	    if (bottom > circuitBottom)
		circuitBottom = bottom;
	}
    }
    
    class FindPathInfo {
	static final int INDUCT  = 1;
	static final int VOLTAGE = 2;
	static final int SHORT   = 3;
	static final int CAP_V   = 4;
	boolean used[];
	int dest;
	CircuitElm firstElm;
	int type;
	FindPathInfo(int t, CircuitElm e, int d) {
	    dest = d;
	    type = t;
	    firstElm = e;
	    used = new boolean[nodeList.size()];
	}
	boolean findPath(int n1) {
	    if (n1 == dest)
		return true;
	    if (used[n1]) {
		//System.out.println("used " + n1);
		return false;
	    }
	    used[n1] = true;
	    int i;
	    for (i = 0; i != elmList.size(); i++) {
		CircuitElm ce = getElm(i);
		if (ce == firstElm)
		    continue;
		if (type == INDUCT) {
		    if (ce instanceof CurrentElm)
			continue;
		}
		if (type == VOLTAGE) {
		    if (!(ce.isWire() || ce instanceof VoltageElm))
			continue;
		}
		if (type == SHORT && !ce.isWire())
		    continue;
		if (type == CAP_V) {
		    if (!(ce.isWire() || ce instanceof CapacitorElm ||
			  ce instanceof VoltageElm))
			continue;
		}
		if (n1 == 0) {
		    // look for posts which have a ground connection;
		    // our path can go through ground
		    int j;
		    for (j = 0; j != ce.getPostCount(); j++)
			if (ce.hasGroundConnection(j) &&
			      findPath(ce.getNode(j))) {
			    used[n1] = false;
			    return true;
			}
		}
		int j;
		for (j = 0; j != ce.getPostCount(); j++) {
		    //System.out.println(ce + " " + ce.getNode(j));
		    if (ce.getNode(j) == n1)
			break;
		}
		if (j == ce.getPostCount())
		    continue;
		if (ce.hasGroundConnection(j) && findPath(0)) {
		    //System.out.println(ce + " has ground");
		    used[n1] = false;
		    return true;
		}
		if (type == INDUCT && ce instanceof InductorElm) {
		    double c = ce.getCurrent();
		    if (j == 0)
			c = -c;
		    //System.out.println("matching " + c + " to " + firstElm.getCurrent());
		    //System.out.println(ce + " " + firstElm);
		    if (Math.abs(c-firstElm.getCurrent()) > 1e-10)
			continue;
		}
		int k;
		for (k = 0; k != ce.getPostCount(); k++) {
		    if (j == k)
			continue;
		    //System.out.println(ce + " " + ce.getNode(j) + "-" + ce.getNode(k));
		    if (ce.getConnection(j, k) && findPath(ce.getNode(k))) {
			//System.out.println("got findpath " + n1);
			used[n1] = false;
			return true;
		    }
		    //System.out.println("back on findpath " + n1);
		}
	    }
	    used[n1] = false;
	    //System.out.println(n1 + " failed");
	    return false;
	}
    }

    void stop(String s, CircuitElm ce) {
	stopMessage = s;
	circuitMatrix = null;
	stopElm = ce;
	stoppedCheck.setState(true);
	analyzeFlag = false;
	cv.repaint();
    }
    
    // stamp independent voltage source #vs, from n1 to n2, amount v
    void stampVoltageSource(int n1, int n2, int vs, double v) {
	int vn = nodeList.size()+vs;
	stampMatrix(vn, n1, -1);
	stampMatrix(vn, n2, 1);
	stampRightSide(vn, v);
	stampMatrix(n1, vn, 1);
	stampMatrix(n2, vn, -1);
    }

    // control voltage source vs with voltage from n1 to n2 (must
    // also call stampVoltageSource())
    void stampVCVS(int n1, int n2, Complex coef, int vs) {
	int vn = nodeList.size()+vs;
	stampMatrix(vn, n1, coef);
	coef.mult(-1);
	stampMatrix(vn, n2, coef);
	coef.mult(-1);
    }
    
    // use this if the amount of voltage is going to be updated in doStep()
    void stampVoltageSource(int n1, int n2, int vs) {
	int vn = nodeList.size()+vs;
	stampMatrix(vn, n1, -1);
	stampMatrix(vn, n2, 1);
	stampRightSide(vn);
	stampMatrix(n1, vn, 1);
	stampMatrix(n2, vn, -1);
    }
    
    void updateVoltageSource(int n1, int n2, int vs, double v) {
	int vn = nodeList.size()+vs;
	stampRightSide(vn, v);
    }
    
    void stampResistor(int n1, int n2, double r) {
	double r0 = 1/r;
	if (Double.isNaN(r0) || Double.isInfinite(r0)) {
	    System.out.print("bad resistance " + r + " " + r0 + "\n");
	    int a = 0;
	    a /= a;
	}
	stampMatrix(n1, n1, r0);
	stampMatrix(n2, n2, r0);
	stampMatrix(n1, n2, -r0);
	stampMatrix(n2, n1, -r0);
    }

    void stampReactance(int n1, int n2, double x) {
	double x0 = 1/x;
	if (Double.isNaN(x0) || Double.isInfinite(x0)) {
	    System.out.print("bad reactance " + x + " " + x0 + "\n");
	    int a = 0;
	    a /= a;
	}
	stampMatrix(n1, n1, 0, x0);
	stampMatrix(n2, n2, 0, x0);
	stampMatrix(n1, n2, 0, -x0);
	stampMatrix(n2, n1, 0, -x0);
    }
    
    void stampConductance(int n1, int n2, double r0) {
	stampMatrix(n1, n1, r0);
	stampMatrix(n2, n2, r0);
	stampMatrix(n1, n2, -r0);
	stampMatrix(n2, n1, -r0);
    }

    // current from cn1 to cn2 is equal to voltage from vn1 to 2, divided by g
    void stampVCCurrentSource(int cn1, int cn2, int vn1, int vn2, double g) {
	stampMatrix(cn1, vn1, g);
	stampMatrix(cn2, vn2, g);
	stampMatrix(cn1, vn2, -g);
	stampMatrix(cn2, vn1, -g);
    }

    void stampCurrentSource(int n1, int n2, double i) {
	stampRightSide(n1, -i);
	stampRightSide(n2, i);
    }

    void stampMatrix(int i, int j, double x) {
	stampMatrix(i, j, x, 0);
    }
    
    void stampMatrix(int i, int j, Complex x) {
	stampMatrix(i, j, x.re, x.im);
    }
    
    // stamp value x in row i, column j, meaning that a voltage change
    // of dv in node j will increase the current into node i by x dv.
    // (Unless i or j is a voltage source node.)
    void stampMatrix(int i, int j, double re, double im) {
	if (i > 0 && j > 0) {
	    if (circuitNeedsMap) {
		i = circuitRowInfo[i-1].mapRow;
		RowInfo ri = circuitRowInfo[j-1];
		if (ri.type == RowInfo.ROW_CONST) {
		    //System.out.println("Stamping constant " + i + " " + j + " " + x);
		    if (im != 0)
			System.out.println("STAMPING COMPLEX CONSTANT!");
		    circuitRightSide[i].add(-re*ri.value, 0);
		    return;
		}
		j = ri.mapCol;
		//System.out.println("stamping " + i + " " + j + " " + x);
	    } else {
		i--;
		j--;
	    }
	    circuitMatrix[i][j].add(re, im);
	}
    }

    // stamp value x on the right side of row i, representing an
    // independent current source flowing into node i
    void stampRightSide(int i, double x) {
	if (i > 0) {
	    if (circuitNeedsMap) {
		i = circuitRowInfo[i-1].mapRow;
		//System.out.println("stamping " + i + " " + x);
	    } else
		i--;
	    circuitRightSide[i].add(x, 0);
	}
    }

    // indicate that the value on the right side of row i changes in doStep()
    void stampRightSide(int i) {
	//System.out.println("rschanges true " + (i-1));
	if (i > 0)
	    circuitRowInfo[i-1].rsChanges = true;
    }
    
    // indicate that the values on the left side of row i change in doStep()
    void stampNonLinear(int i) {
	if (i > 0)
	    circuitRowInfo[i-1].lsChanges = true;
    }

    double getIterCount() {
	//return (Math.exp((speedBar.getValue()-1)/24.) + .5);
	return .1*Math.exp((speedBar.getValue()-61)/24.);
    }
    
    int min(int a, int b) { return (a < b) ? a : b; }
    int max(int a, int b) { return (a > b) ? a : b; }

    void editFuncPoint(int x, int y) {
	// XXX
	cv.repaint(pause);
    }

    public void componentHidden(ComponentEvent e){}
    public void componentMoved(ComponentEvent e){}
    public void componentShown(ComponentEvent e) {
	cv.repaint();
    }

    public void componentResized(ComponentEvent e) {
	handleResize();
	cv.repaint(100);
    }
    public void actionPerformed(ActionEvent e) {
	String ac = e.getActionCommand();
	if (e.getSource() == resetButton)
	    reset();
	if (e.getSource() == dumpMatrixButton)
	    dumpMatrix = true;
	if (e.getSource() == exportItem)
	    doImport(false);
	if (e.getSource() == optionsItem)
	    doEdit(new EditOptions(this));
	if (e.getSource() == importItem)
	    doImport(true);
	if (e.getSource() == undoItem)
	    doUndo();
	if (e.getSource() == redoItem)
	    doRedo();
	if (ac.compareTo("Cut") == 0) {
	    if (e.getSource() != elmCutMenuItem)
		menuElm = null;
	    doCut();
	}
	if (ac.compareTo("Copy") == 0) {
	    if (e.getSource() != elmCopyMenuItem)
		menuElm = null;
	    doCopy();
	}
	if (ac.compareTo("Paste") == 0)
	    doPaste();
	if (e.getSource() == selectAllItem)
	    doSelectAll();
	if (e.getSource() == exitItem) {
	    applet.destroyFrame();
	    return;
	}
	if (ac.compareTo("stackAll") == 0)
	    stackAll();
	if (ac.compareTo("unstackAll") == 0)
	    unstackAll();
	if (e.getSource() == elmEditMenuItem)
	    doEdit(menuElm);
	if (ac.compareTo("Delete") == 0) {
	    if (e.getSource() != elmDeleteMenuItem)
		menuElm = null;
	    doDelete();
	}
	if (e.getSource() == elmScopeMenuItem && menuElm != null) {
	    int i;
	    for (i = 0; i != scopeCount; i++)
		if (scopes[i].elm == null)
		    break;
	    if (i == scopeCount) {
		if (scopeCount == scopes.length)
		    return;
		scopeCount++;
		scopes[i] = new Scope(this);
		scopes[i].position = i;
		handleResize();
	    }
	    scopes[i].setElm(menuElm);
	}
	if (menuScope != -1) {
	    if (ac.compareTo("remove") == 0)
		scopes[menuScope].setElm(null);
	    if (ac.compareTo("speed2") == 0)
		scopes[menuScope].speedUp();
	    if (ac.compareTo("speed1/2") == 0)
		scopes[menuScope].slowDown();
	    if (ac.compareTo("scale") == 0)
		scopes[menuScope].adjustScale(.5);
	    if (ac.compareTo("maxscale") == 0)
		scopes[menuScope].adjustScale(1e-50);
	    if (ac.compareTo("stack") == 0)
		stackScope(menuScope);
	    if (ac.compareTo("unstack") == 0)
		unstackScope(menuScope);
	    if (ac.compareTo("selecty") == 0)
		scopes[menuScope].selectY();
	    if (ac.compareTo("reset") == 0)
		scopes[menuScope].resetGraph();
	    cv.repaint();
	}
	if (ac.indexOf("setup ") == 0) {
	    pushUndo();
	    readSetupFile(ac.substring(6),
			  ((MenuItem) e.getSource()).getLabel());
	}
    }

    void reset() {
	int i;
	
	// on IE, drawImage() stops working inexplicably every once in
	// a while.  Recreating it fixes the problem, so we do that here.
	dbimage = main.createImage(winSize.width, winSize.height);
	
	for (i = 0; i != elmList.size(); i++)
	    getElm(i).reset();
	for (i = 0; i != scopeCount; i++)
	    scopes[i].resetGraph();
	//analyzeFlag = true;
	t = 0;
	lastIterTime = 0;
	stoppedCheck.setState(false);
	cv.repaint();
    }
    
    void stackScope(int s) {
	if (s == 0) {
	    if (scopeCount < 2)
		return;
	    s = 1;
	}
	if (scopes[s].position == scopes[s-1].position)
	    return;
	scopes[s].position = scopes[s-1].position;
	for (s++; s < scopeCount; s++)
	    scopes[s].position--;
    }
    
    void unstackScope(int s) {
	if (s == 0) {
	    if (scopeCount < 2)
		return;
	    s = 1;
	}
	if (scopes[s].position != scopes[s-1].position)
	    return;
	for (; s < scopeCount; s++)
	    scopes[s].position++;
    }

    void stackAll() {
	int i;
	for (i = 0; i != scopeCount; i++) {
	    scopes[i].position = 0;
	    scopes[i].showMax = scopes[i].showMin = false;
	}
    }

    void unstackAll() {
	int i;
	for (i = 0; i != scopeCount; i++) {
	    scopes[i].position = i;
	    scopes[i].showMax = true;
	}
    }
    
    void doEdit(Editable eable) {
	clearSelection();
	pushUndo();
	if (editDialog != null) {
	    requestFocus();
	    editDialog.setVisible(false);
	    editDialog = null;
	}
	editDialog = new EditDialog(eable, this);
	editDialog.show();
    }

    void doImport(boolean imp) {
	if (impDialog != null) {
	    requestFocus();
	    impDialog.setVisible(false);
	    impDialog = null;
	}
	String dump = (imp) ? "" : dumpCircuit();
	impDialog = new ImportDialog(this, dump);
	impDialog.show();
	pushUndo();
    }
    
    String dumpCircuit() {
	int i;
	int f = (dotsCheckItem.getState()) ? 1 : 0;
	f |= (smallGridCheckItem.getState()) ? 2 : 0;
	//f |= (voltsCheckItem.getState()) ? 0 : 4;
	f |= (powerCheckItem.getState()) ? 8 : 0;
	f |= (showValuesCheckItem.getState()) ? 0 : 16;
	// 32 = linear scale in afilter
	String dump = "$ " + f + " " +
	    timeStep + " " + 5 + " " +
	    currentBar.getValue() + " " + CircuitElm.voltageRange + " " +
	    powerBar.getValue() + "\n";
	f = 0;
	f |= (linearCheckItem.getState()) ? 1 : 0;
	f |= (polesCheckItem.getState()) ? 2 : 0;
	f |= (phaseCheckItem.getState()) ? 4 : 0;
	dump += "% " + f + " " + maxFrequency + "\n";
	for (i = 0; i != elmList.size(); i++)
	    dump += getElm(i).dump() + "\n";
	for (i = 0; i != scopeCount; i++) {
	    String d = scopes[i].dump();
	    if (d != null)
		dump += d + "\n";
	}
	if (hintType != -1)
	    dump += "h " + hintType + " " + hintItem1 + " " +
		hintItem2 + "\n";
	return dump;
    }
    
    public void adjustmentValueChanged(AdjustmentEvent e) {
	System.out.print(((Scrollbar) e.getSource()).getValue() + "\n");
	if (e.getSource() == scrollXBar) {
	    int newminx = -scrollXBar.getValue();
	    int oldminx = circuitBbox.x;
	    int dx = snapGrid(newminx-oldminx);
	    moveAll(dx, 0);
	}
	if (e.getSource() == zoomBar)
	    setupResponse();
	if (customizer != null)
	    customizer.slidersChanged((Scrollbar) e.getSource());
    }

    void setupResponse() {
	// default = 25000, range = 1000 .. 20Ghz
	double log1 = Math.log(1000/minFrequency);
	double log2 = Math.log(2e10/minFrequency);
	maxFrequency = minFrequency *
	    Math.exp(log1+(log2-log1)*zoomBar.getValue()/100.);
	freqLogRange = Math.log(maxFrequency/minFrequency);
	minLogFrequency = Math.log(minFrequency);
	analyzeFlag = true;
    }
    
    ByteArrayOutputStream readUrlData(URL url) throws java.io.IOException {
	Object o = url.getContent();
	FilterInputStream fis = (FilterInputStream) o;
	ByteArrayOutputStream ba = new ByteArrayOutputStream(fis.available());
	int blen = 1024;
	byte b[] = new byte[blen];
	while (true) {
	    int len = fis.read(b);
	    if (len <= 0)
		break;
	    ba.write(b, 0, len);
	}
	return ba;
    }
    
    void getSetupList(Menu menu, boolean retry) {
	Menu stack[] = new Menu[6];
	int stackptr = 0;
	stack[stackptr++] = menu;
	try {
	    URL url = new URL(applet.getCodeBase() + "setuplist.txt");
	    ByteArrayOutputStream ba = readUrlData(url);
	    byte b[] = ba.toByteArray();
	    int len = ba.size();
	    int p;
	    if (len == 0 || b[0] != '#') {
		// got a redirect, try again
		getSetupList(menu, true);
		return;
	    }
	    for (p = 0; p < len; ) {
		int l;
		for (l = 0; l != len-p; l++)
		    if (b[l+p] == '\n') {
			l++;
			break;
		    }
		String line = new String(b, p, l-1);
		if (line.charAt(0) == '#')
		    ;
		else if (line.charAt(0) == '+') {
		    Menu n = new Menu(line.substring(1));
		    menu.add(n);
		    menu = stack[stackptr++] = n;
		} else if (line.charAt(0) == '-') {
		    menu = stack[--stackptr-1];
		} else {
		    int i = line.indexOf(' ');
		    if (i > 0)
			menu.add(getMenuItem(line.substring(i+1),
					     "setup " + line.substring(0, i)));
		}
		p += l;
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	    stop("Can't read setuplist.txt!", null);
	}
    }

    void readSetup(String text) {
	readSetup(text, false);
    }
    
    void readSetup(String text, boolean retain) {
	readSetup(text.getBytes(), text.length(), retain);
	titleLabel.setText("untitled");
    }

    void readSetupFile(String str, String title) {
	t = 0;
	System.out.println(str);
	try {
	    URL url = new URL(applet.getCodeBase() + "circuits/" + str);
	    ByteArrayOutputStream ba = readUrlData(url);
	    readSetup(ba.toByteArray(), ba.size(), false);
	} catch (Exception e) {
	    e.printStackTrace();
	    stop("Unable to read " + str + "!", null);
	}
	titleLabel.setText(title);
    }

    void readSetup(byte b[], int len, boolean retain) {
	int i;
	if (!retain) {
	    for (i = 0; i != elmList.size(); i++) {
		CircuitElm ce = getElm(i);
		ce.delete();
	    }
	    elmList.removeAllElements();
	    releaseCustomizer();
	    hintType = -1;
	    timeStep = 5e-6;
	    dotsCheckItem.setState(true);
	    smallGridCheckItem.setState(false);
	    powerCheckItem.setState(false);
	    voltsCheckItem.setState(true);
	    showValuesCheckItem.setState(true);
	    setGrid();
	    speedBar.setValue(117); // 57
	    currentBar.setValue(50);
	    powerBar.setValue(50);
	    zoomBar.setValue(20);
	    linearCheckItem.setState(false);
	    CircuitElm.voltageRange = 5;
	    scopeCount = 0;
	}
	cv.repaint();
	int p;
	for (p = 0; p < len; ) {
	    int l;
	    int linelen = 0;
	    for (l = 0; l != len-p; l++)
		if (b[l+p] == '\n' || b[l+p] == '\r') {
		    linelen = l++;
		    if (l+p < b.length && b[l+p] == '\n')
			l++;
		    break;
		}
	    String line = new String(b, p, linelen);
	    try {
		undump(line);
	    } catch (java.lang.reflect.InvocationTargetException ee) {
		ee.getTargetException().printStackTrace();
	    } catch (Exception ee) {
		ee.printStackTrace();
	    }
	    p += l;
	}
 	if (customizer != null)
 	    customizer.create();
 	animateFreq = 0;
 	setVPACheck(admittanceCheckItem);
	enableItems();
	setupResponse();
	if (!retain)
	    handleResize(); // for scopes
	needAnalyze();
    }

    CircuitElm undump(String line) throws Exception {
	StringTokenizer st = new StringTokenizer(line);
	if (!st.hasMoreTokens())
	    return null;
	String type = st.nextToken();
	int tint = type.charAt(0);
	if (tint == 'o') {
	    Scope sc = new Scope(this);
	    sc.position = scopeCount;
	    sc.undump(st);
	    scopes[scopeCount++] = sc;
	    return null;
	}
	if (tint == 'h') {
	    readHint(st);
	    return null;
	}
	if (tint == '?') {
	    readCustomizer(st);
	    return null;
	}
	if (tint == '$') {
	    readOptions(st);
	    return null;
	}
	if (tint == '%') {
	    readFreqOptions(st);
	    return null;
	}
	if (tint >= '0' && tint <= '9')
	    tint = new Integer(type).intValue();
	int x1 = new Integer(st.nextToken()).intValue();
	int y1 = new Integer(st.nextToken()).intValue();
	int x2 = new Integer(st.nextToken()).intValue();
	int y2 = new Integer(st.nextToken()).intValue();
	int f  = new Integer(st.nextToken()).intValue();
	CircuitElm ce = null;
	Class cls = dumpTypes[tint];
	if (cls == null) {
	    System.out.println("unrecognized dump type: " + type);
	    return null;
	}
	// find element class
	Class carr[] = new Class[6];
	//carr[0] = getClass();
	carr[0] = carr[1] = carr[2] = carr[3] = carr[4] =
	    int.class;
	carr[5] = StringTokenizer.class;
	Constructor cstr = null;
	cstr = cls.getConstructor(carr);
	
	// invoke constructor with starting coordinates
	Object oarr[] = new Object[6];
	//oarr[0] = this;
	oarr[0] = new Integer(x1);
	oarr[1] = new Integer(y1);
	oarr[2] = new Integer(x2);
	oarr[3] = new Integer(y2);
	oarr[4] = new Integer(f );
	oarr[5] = st;
	ce = (CircuitElm) cstr.newInstance(oarr);
	ce.setPoints();
	elmList.addElement(ce);
	return ce;
    }
    
    void readHint(StringTokenizer st) {
	hintType  = new Integer(st.nextToken()).intValue();
	hintItem1 = new Integer(st.nextToken()).intValue();
	hintItem2 = new Integer(st.nextToken()).intValue();
    }
    
    void readCustomizer(StringTokenizer st) {
 	int flags = new Integer(st.nextToken()).intValue();
 	String s = st.nextToken();
 	Constructor cstr = null;
 	Object o = null;
 	try {
 	    Class c = Class.forName(s);
 	    cstr = c.getConstructor(new Class[] { int.class,
						  StringTokenizer.class });
 	    o = cstr.newInstance(new Object[] { new Integer(flags), st });
 	} catch (Exception ee) {
 	    ee.printStackTrace();
 	    return;
 	}
 	customizer = (Customizer) o;
    }
    
    void releaseCustomizer() {
 	if (customizer != null) {
 	    customizer.delete();
 	    customizer = null;
 	}
    }
    
    void readOptions(StringTokenizer st) {
	int flags = new Integer(st.nextToken()).intValue();
	dotsCheckItem.setState((flags & 1) != 0);
	smallGridCheckItem.setState((flags & 2) != 0);
	voltsCheckItem.setState((flags & 4) == 0);
	powerCheckItem.setState((flags & 8) == 8);
	showValuesCheckItem.setState((flags & 16) == 0);
	timeStep = new Double (st.nextToken()).doubleValue();
	double sp = new Double(st.nextToken()).doubleValue();
	int sp2 = (int) (Math.log(10*sp)*24+61.5);
	//int sp2 = (int) (Math.log(sp)*24+1.5);
	speedBar  .setValue(sp2);
	currentBar.setValue(new Integer(st.nextToken()).intValue());
	CircuitElm.voltageRange = new Double (st.nextToken()).doubleValue();
	try {
	    powerBar.setValue(new Integer(st.nextToken()).intValue());
	} catch (Exception e) {
	}
	setGrid();
    }

    void readFreqOptions(StringTokenizer st) {
	int flags = new Integer(st.nextToken()).intValue();
	linearCheckItem.setState((flags & 1) != 0);
	//polesCheckItem.setState((flags & 2) != 0);
	phaseCheckItem.setState((flags & 4) != 0);
	double mf = new Double(st.nextToken()).doubleValue();
	double log1 = Math.log(1000/minFrequency);
	double log2 = Math.log(2e10/minFrequency);
	int mfi = (int) ((Math.log(mf/minFrequency)-log1)*100/(log2-log1));
	zoomBar.setValue(mfi);
    }
    
    int snapGrid(int x) {
	return (x+gridRound) & gridMask;
    }

    boolean doSwitch(int x, int y) {
	if (mouseElm == null || !(mouseElm instanceof SwitchElm))
	    return false;
	SwitchElm se = (SwitchElm) mouseElm;
	se.toggle();
	if (se.momentary)
	    heldSwitchElm = se;
	needAnalyze();
	return true;
    }

    int locateElm(CircuitElm elm) {
	int i;
	for (i = 0; i != elmList.size(); i++)
	    if (elm == elmList.elementAt(i))
		return i;
	return -1;
    }
    
    public void mouseDragged(MouseEvent e) {
	// ignore right mouse button with no modifiers (needed on PC)
	if ((e.getModifiers() & MouseEvent.BUTTON3_MASK) != 0) {
	    int ex = e.getModifiersEx();
	    if ((ex & (MouseEvent.META_DOWN_MASK|
		       MouseEvent.SHIFT_DOWN_MASK|
		       MouseEvent.CTRL_DOWN_MASK|
		       MouseEvent.ALT_DOWN_MASK)) == 0)
		return;
	}
	if (!circuitArea.contains(e.getX(), e.getY()))
	    return;
	if (dragElm != null)
	    dragElm.drag(e.getX(), e.getY());
	boolean success = true;
	switch (tempMouseMode) {
	case MODE_DRAG_ALL:
	    dragAll(snapGrid(e.getX()), snapGrid(e.getY()));
	    break;
	case MODE_DRAG_ROW:
	    dragRow(snapGrid(e.getX()), snapGrid(e.getY()));
	    break;
	case MODE_DRAG_COLUMN:
	    dragColumn(snapGrid(e.getX()), snapGrid(e.getY()));
	    break;
	case MODE_DRAG_POST:
	    if (mouseElm != null)
		dragPost(snapGrid(e.getX()), snapGrid(e.getY()));
	    break;
	case MODE_SELECT:
	    if (mouseElm == null)
		selectArea(e.getX(), e.getY());
	    else {
		tempMouseMode = MODE_DRAG_SELECTED;
		success = dragSelected(e.getX(), e.getY());
	    }
	    break;
	case MODE_DRAG_SELECTED:
	    success = dragSelected(e.getX(), e.getY());
	    break;
	}
	dragging = true;
	if (success) {
	    if (tempMouseMode == MODE_DRAG_SELECTED && mouseElm instanceof TextElm) {
		dragX = e.getX(); dragY = e.getY();
	    } else {
		dragX = snapGrid(e.getX()); dragY = snapGrid(e.getY());
	    }
	}
	cv.repaint(pause);
    }

    void dragAll(int x, int y) {
	int dx = x-dragX;
	int dy = y-dragY;
	moveAll(dx, dy);
    }

    void moveAll(int dx, int dy) {
	if (dx == 0 && dy == 0)
	    return;
	int i;
	for (i = 0; i != elmList.size(); i++) {
	    CircuitElm ce = getElm(i);
	    ce.move(dx, dy);
	}
	removeZeroLengthElements();
	circuitBbox.translate(dx, dy);
    }

    void makeRoom(int x, int dx) {
	int i;
	for (i = 0; i != elmList.size(); i++) {
	    CircuitElm ce = getElm(i);
	    if (ce.x > x || ce.x2 > x)
		ce.move(dx, 0);
	}
    }

    BoxElm cloneBox(BoxElm be, int dx) {
	int i;
	int size = elmList.size();
	BoxElm newbox = (BoxElm) clone(be);
	newbox.move(dx, 0);
	for (i = 0; i != size; i++) {
	    CircuitElm ce = getElm(i);
	    if (ce == be || ce == newbox)
		continue;
	    if (ce.x == be.x && ce.x2 == be.x)
		continue;
	    if ((ce.x  >= be.x && ce.x  <= be.x2) &&
		(ce.x2 >= be.x && ce.x2 <= be.x2)) {
		CircuitElm cn = clone(ce);
		cn.move(dx, 0);
	    }
	}
	return newbox;
    }

    void deleteBox(CircuitElm be) {
	int i;
	for (i = 0; i != elmList.size(); i++) {
	    CircuitElm ce = getElm(i);
	    if (ce.x == be.x && ce.x2 == be.x)
		continue;
	    if ((ce.x  >= be.x && ce.x  <= be.x2) &&
		(ce.x2 >= be.x && ce.x2 <= be.x2)) {
		//System.out.println("deleting " + ce);
		ce.delete();
		elmList.removeElementAt(i--);
	    }
	}
    }
    
    CircuitElm clone(CircuitElm ce) {
	String s = ce.dump();
	try {
	    return undump(s);
	} catch (Exception e) {
	    e.printStackTrace();
	    return null;
	}
    }
    
    void dragRow(int x, int y) {
	int dy = y-dragY;
	if (dy == 0)
	    return;
	int i;
	for (i = 0; i != elmList.size(); i++) {
	    CircuitElm ce = getElm(i);
	    if (ce.y  == dragY)
		ce.movePoint(0, 0, dy);
	    if (ce.y2 == dragY)
		ce.movePoint(1, 0, dy);
	}
	removeZeroLengthElements();
    }
    
    void dragColumn(int x, int y) {
	int dx = x-dragX;
	if (dx == 0)
	    return;
	int i;
	for (i = 0; i != elmList.size(); i++) {
	    CircuitElm ce = getElm(i);
	    if (ce.x  == dragX)
		ce.movePoint(0, dx, 0);
	    if (ce.x2 == dragX)
		ce.movePoint(1, dx, 0);
	}
	removeZeroLengthElements();
    }

    boolean dragSelected(int x, int y) {
	boolean me = false;
	if (mouseElm != null && !mouseElm.isSelected())
	    mouseElm.setSelected(me = true);

	// snap grid, unless we're only dragging text elements
	int i;
	for (i = 0; i != elmList.size(); i++) {
	    CircuitElm ce = getElm(i);
	    if (ce.isSelected() && !(ce instanceof TextElm))
		break;
	}
	if (i != elmList.size()) {
	    x = snapGrid(x);
	    y = snapGrid(y);
	}
	
	int dx = x-dragX;
	int dy = y-dragY;
	if (dx == 0 && dy == 0) {
	    // don't leave mouseElm selected if we selected it above
	    if (me)
		mouseElm.setSelected(false);
	    return false;
	}
	boolean allowed = true;

	// check if moves are allowed
	for (i = 0; allowed && i != elmList.size(); i++) {
	    CircuitElm ce = getElm(i);
	    if (ce.isSelected() && !ce.allowMove(dx, dy))
		allowed = false;
	}

	if (allowed) {
	    for (i = 0; i != elmList.size(); i++) {
		CircuitElm ce = getElm(i);
		if (ce.isSelected())
		    ce.move(dx, dy);
	    }
	    needAnalyze();
	}
	
	// don't leave mouseElm selected if we selected it above
	if (me)
	    mouseElm.setSelected(false);
	
	return allowed;
    }

    void dragPost(int x, int y) {
	if (draggingPost == -1) {
	    draggingPost =
		(distanceSq(mouseElm.x , mouseElm.y , x, y) >
		 distanceSq(mouseElm.x2, mouseElm.y2, x, y)) ? 1 : 0;
	}
	int dx = x-dragX;
	int dy = y-dragY;
	if (dx == 0 && dy == 0)
	    return;
	mouseElm.movePoint(draggingPost, dx, dy);
	needAnalyze();
    }

    void selectArea(int x, int y) {
	int x1 = min(x, initDragX);
	int x2 = max(x, initDragX);
	int y1 = min(y, initDragY);
	int y2 = max(y, initDragY);
	selectedArea = new Rectangle(x1, y1, x2-x1, y2-y1);
	int i;
	for (i = 0; i != elmList.size(); i++) {
	    CircuitElm ce = getElm(i);
	    ce.selectRect(selectedArea);
	}
    }

    void setSelectedElm(CircuitElm cs) {
	int i;
	for (i = 0; i != elmList.size(); i++) {
	    CircuitElm ce = getElm(i);
	    ce.setSelected(ce == cs);
	}
	mouseElm = cs;
    }
    
    void removeZeroLengthElements() {
	int i;
	boolean changed = false;
	for (i = elmList.size()-1; i >= 0; i--) {
	    CircuitElm ce = getElm(i);
	    if (ce.x == ce.x2 && ce.y == ce.y2) {
		elmList.removeElementAt(i);
		ce.delete();
		changed = true;
	    }
	}
	if (changed)
	    needAnalyze();
    }

    public void mouseMoved(MouseEvent e) {
	if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) != 0)
	    return;
	int x = e.getX();
	int y = e.getY();
	dragX = snapGrid(x); dragY = snapGrid(y);
	draggingPost = -1;
	int i;
	CircuitElm origMouse = mouseElm;
	mouseElm = null;
	selectedFreq = 0;
	mousePost = -1;
	plotXElm = plotYElm = null;
	int bestDist = 10000000;
	int bestArea = 10000000;
	for (i = 0; i != elmList.size(); i++) {
	    CircuitElm ce = getElm(i);
	    if (ce.boundingBox.contains(x, y)) {
		int j;
		int area = ce.boundingBox.width * ce.boundingBox.height;
		int jn = ce.getPostCount();
		if (jn > 2)
		    jn = 2;
		for (j = 0; j != jn; j++) {
		    Point pt = ce.getPost(j);
		    int dist = distanceSq(x, y, pt.x, pt.y);

		    // if multiple elements have overlapping bounding boxes,
		    // we prefer selecting elements that have posts close
		    // to the mouse pointer and that have a small bounding
		    // box area.
		    if (dist <= bestDist && area <= bestArea) {
			bestDist = dist;
			bestArea = area;
			mouseElm = ce;
		    }
		}
		if (ce.getPostCount() == 0 && area <= bestArea) {
		    bestArea = area;
		    mouseElm = ce;
		}
	    }
	}
	scopeSelected = -1;
	double oldFreq = selectedFreq;
	if (mouseElm == null) {
	    if (responseArea.contains(x, y))
		selectedFreq = linearToFrequency(x/(double) responseArea.width);
	    // the mouse pointer was not in any of the bounding boxes, but we
	    // might still be close to a post
	    for (i = 0; i != elmList.size(); i++) {
		CircuitElm ce = getElm(i);
		int j;
		int jn = ce.getPostCount();
		for (j = 0; j != jn; j++) {
		    Point pt = ce.getPost(j);
		    int dist = distanceSq(x, y, pt.x, pt.y);
		    if (distanceSq(pt.x, pt.y, x, y) < 26) {
			mouseElm = ce;
			mousePost = j;
			break;
		    }
		}
	    }
	} else {
	    mousePost = -1;
	    // look for post close to the mouse pointer
	    for (i = 0; i != mouseElm.getPostCount(); i++) {
		Point pt = mouseElm.getPost(i);
		if (distanceSq(pt.x, pt.y, x, y) < 26)
		    mousePost = i;
	    }
	}
	if (mouseElm != origMouse || oldFreq != selectedFreq)
	    cv.repaint();
    }

    int distanceSq(int x1, int y1, int x2, int y2) {
	x2 -= x1;
	y2 -= y1;
	return x2*x2+y2*y2;
    }

    public void mouseClicked(MouseEvent e) {
	if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) != 0) {
	    if (mouseMode == MODE_SELECT || mouseMode == MODE_DRAG_SELECTED)
		clearSelection();
	}
    }
    public void mouseEntered(MouseEvent e) {
    }
    public void mouseExited(MouseEvent e) {
	scopeSelected = -1;
	mouseElm = plotXElm = plotYElm = null;
	cv.repaint();
    }
    
    public void mousePressed(MouseEvent e) {
	System.out.println(e.getModifiers());
	int ex = e.getModifiersEx();
	if ((ex & (MouseEvent.META_DOWN_MASK|
		   MouseEvent.SHIFT_DOWN_MASK)) == 0 && e.isPopupTrigger()) {
	    doPopupMenu(e);
	    return;
	}
	if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) != 0) {
	    // left mouse
	    tempMouseMode = mouseMode;
	    if ((ex & MouseEvent.ALT_DOWN_MASK) != 0 &&
		(ex & MouseEvent.META_DOWN_MASK) != 0)
		tempMouseMode = MODE_DRAG_COLUMN;
	    else if ((ex & MouseEvent.ALT_DOWN_MASK) != 0 &&
		     (ex & MouseEvent.SHIFT_DOWN_MASK) != 0)
		tempMouseMode = MODE_DRAG_ROW;
	    else if ((ex & MouseEvent.SHIFT_DOWN_MASK) != 0)
		tempMouseMode = MODE_SELECT;
	    else if ((ex & MouseEvent.ALT_DOWN_MASK) != 0)
		tempMouseMode = MODE_DRAG_ALL;
	    else if ((ex & (MouseEvent.CTRL_DOWN_MASK|
			    MouseEvent.META_DOWN_MASK)) != 0)
		tempMouseMode = MODE_DRAG_POST;
	} else if ((e.getModifiers() & MouseEvent.BUTTON3_MASK) != 0) {
	    // right mouse
	    if ((ex & MouseEvent.SHIFT_DOWN_MASK) != 0)
		tempMouseMode = MODE_DRAG_ROW;
	    else if ((ex & (MouseEvent.CTRL_DOWN_MASK|
			    MouseEvent.META_DOWN_MASK)) != 0)
		tempMouseMode = MODE_DRAG_COLUMN;
	    else
		return;
	}
	
	if (tempMouseMode != MODE_SELECT && tempMouseMode != MODE_DRAG_SELECTED)
	    clearSelection();
	if (doSwitch(e.getX(), e.getY()))
	    return;
	if (animateFreq > 0 && selectedFreq == 0) {
	    reset();
	    setVPACheck(admittanceCheckItem);
	}
	if (selectedFreq > 0 && animateFreq == 0)
	    setVPACheck(voltsCheckItem);
	animateFreq = selectedFreq;
	t = 0;
	setAnimateSpeed();
	pushUndo();
	initDragX = e.getX();
	initDragY = e.getY();
	dragging = true;
	if (tempMouseMode != MODE_ADD_ELM || addingClass == null)
	    return;
	
	int x0 = snapGrid(e.getX());
	int y0 = snapGrid(e.getY());
	if (!circuitArea.contains(x0, y0))
	    return;

	dragElm = constructElement(addingClass, x0, y0);
    }

    void setAnimateSpeed() {
	speedBar.setValue(speedBar.getMinimum());
	while (true) {
	    double steprate = 160*getIterCount();
	    double x = steprate*timeStep*animateFreq*2;
	    if (x > 1)
		break;
	    int next = speedBar.getValue()+32;
	    if (next > speedBar.getMaximum())
		break;
	    speedBar.setValue(next);
	}
    }
    
    CircuitElm constructElement(Class c, int x0, int y0) {
	// find element class
	Class carr[] = new Class[2];
	//carr[0] = getClass();
	carr[0] = carr[1] = int.class;
	Constructor cstr = null;
	try {
	    cstr = c.getConstructor(carr);
	} catch (NoSuchMethodException ee) {
	    System.out.println("caught NoSuchMethodException");
	    return null;
	} catch (Exception ee) {
	    ee.printStackTrace();
	    return null;
	}

	// invoke constructor with starting coordinates
	Object oarr[] = new Object[2];
	oarr[0] = new Integer(x0);
	oarr[1] = new Integer(y0);
	try {
	    return (CircuitElm) cstr.newInstance(oarr);
	} catch (Exception ee) { ee.printStackTrace(); }
	return null;
    }
    
    void doPopupMenu(MouseEvent e) {
	menuElm = mouseElm;
	menuScope = -1;
	if (scopeSelected != -1) {
	    PopupMenu m = scopes[scopeSelected].getMenu();
	    menuScope = scopeSelected;
	    if (m != null)
		m.show(e.getComponent(), e.getX(), e.getY());
	} else if (mouseElm != null) {
	    elmEditMenuItem .setEnabled(mouseElm.getEditInfo(0) != null);
	    elmScopeMenuItem.setEnabled(mouseElm.canViewInScope());
	    elmMenu.show(e.getComponent(), e.getX(), e.getY());
	} else {
	    doMainMenuChecks(mainMenu);
	    mainMenu.show(e.getComponent(), e.getX(), e.getY());
	}
    }

    void doMainMenuChecks(Menu m) {
	int i;
	if (m == optionsMenu)
	    return;
	for (i = 0; i != m.getItemCount(); i++) {
	    MenuItem mc = m.getItem(i);
	    if (mc instanceof Menu)
		doMainMenuChecks((Menu) mc);
	    if (mc instanceof CheckboxMenuItem) {
		CheckboxMenuItem cmi = (CheckboxMenuItem) mc;
		cmi.setState(
		      mouseModeStr.compareTo(cmi.getActionCommand()) == 0);
	    }
	}
    }
    
    public void mouseReleased(MouseEvent e) {
	int ex = e.getModifiersEx();
	if ((ex & (MouseEvent.SHIFT_DOWN_MASK|MouseEvent.CTRL_DOWN_MASK|
		   MouseEvent.META_DOWN_MASK)) == 0 && e.isPopupTrigger()) {
	    doPopupMenu(e);
	    return;
	}
	tempMouseMode = mouseMode;
	selectedArea = null;
	dragging = false;
	boolean circuitChanged = false;
	if (heldSwitchElm != null) {
	    heldSwitchElm.mouseUp();
	    heldSwitchElm = null;
	    circuitChanged = true;
	}
	if (dragElm != null) {
	    // if the element is zero size then don't create it
	    if (dragElm.x == dragElm.x2 && dragElm.y == dragElm.y2)
		dragElm.delete();
	    else {
		elmList.addElement(dragElm);
		releaseCustomizer();
		circuitChanged = true;
	    }
	    dragElm = null;
	}
	if (circuitChanged)
	    needAnalyze();
	if (dragElm != null)
	    dragElm.delete();
	dragElm = null;
	cv.repaint();
    }

    void enableItems() {
	if (powerCheckItem.getState()) {
	    powerBar.enable();
	    powerLabel.enable();
	} else {
	    powerBar.disable();
	    powerLabel.disable();
	}
	enableUndoRedo();
    }
    
    public void itemStateChanged(ItemEvent e) {
	cv.repaint(pause);
	Object mi = e.getItemSelectable();
	if (mi == stoppedCheck)
	    return;
	if (mi == smallGridCheckItem)
	    setGrid();
	if (mi == polesCheckItem)
	    handleResize();
	if (mi == linearCheckItem)
	    analyzeFlag = true;
	if (mi == powerCheckItem || mi == voltsCheckItem ||
	    mi == admittanceCheckItem) {
	    CheckboxMenuItem cmi = (CheckboxMenuItem) mi;
	    if (cmi.getState())
		setVPACheck(cmi);
	    else if (cmi != voltsCheckItem)
		setVPACheck(voltsCheckItem);
	}
	enableItems();
	if (menuScope != -1) {
	    Scope sc = scopes[menuScope];
	    sc.handleMenu(e, mi);
	}
	if (mi instanceof CheckboxMenuItem) {
	    MenuItem mmi = (MenuItem) mi;
	    mouseMode = MODE_ADD_ELM;
	    String s = mmi.getActionCommand();
	    if (s.length() > 0)
		mouseModeStr = s;
	    if (s.compareTo("DragAll") == 0)
		mouseMode = MODE_DRAG_ALL;
	    else if (s.compareTo("DragRow") == 0)
		mouseMode = MODE_DRAG_ROW;
	    else if (s.compareTo("DragColumn") == 0)
		mouseMode = MODE_DRAG_COLUMN;
	    else if (s.compareTo("DragSelected") == 0)
		mouseMode = MODE_DRAG_SELECTED;
	    else if (s.compareTo("DragPost") == 0)
		mouseMode = MODE_DRAG_POST;
	    else if (s.compareTo("Select") == 0)
		mouseMode = MODE_SELECT;
	    else if (s.length() > 0) {
		try {
		    addingClass = Class.forName(s);
		} catch (Exception ee) {
		    ee.printStackTrace();
		}
	    }
	    tempMouseMode = mouseMode;
	}
    }

    void setVPACheck(CheckboxMenuItem mi) {
	voltsCheckItem.setState(mi == voltsCheckItem);
	powerCheckItem.setState(mi == powerCheckItem);
	admittanceCheckItem.setState(mi == admittanceCheckItem);
    }
    
    void setGrid() {
	gridSize = (smallGridCheckItem.getState()) ? 8 : 16;
	gridMask = ~(gridSize-1);
	gridRound = gridSize/2-1;
    }

    void pushUndo() {
	redoStack.removeAllElements();
	String s = dumpCircuit();
	if (undoStack.size() > 0 && s.compareTo((String) (undoStack.lastElement())) == 0)
	    return;
	undoStack.add(s);
	enableUndoRedo();
    }

    void doUndo() {
	if (undoStack.size() == 0)
	    return;
	redoStack.add(dumpCircuit());
	String s = (String) (undoStack.remove(undoStack.size()-1));
	readSetup(s);
	enableUndoRedo();
    }

    void doRedo() {
	if (redoStack.size() == 0)
	    return;
	undoStack.add(dumpCircuit());
	String s = (String) (redoStack.remove(redoStack.size()-1));
	readSetup(s);
	enableUndoRedo();
    }

    void enableUndoRedo() {
	redoItem.setEnabled(redoStack.size() > 0);
	undoItem.setEnabled(undoStack.size() > 0);
    }

    void setMenuSelection() {
	if (menuElm != null) {
	    if (menuElm.selected)
		return;
	    clearSelection();
	    menuElm.setSelected(true);
	}
    }
    
    void doCut() {
	int i;
	pushUndo();
	setMenuSelection();
	clipboard = "";
	for (i = elmList.size()-1; i >= 0; i--) {
	    CircuitElm ce = getElm(i);
	    if (ce.isSelected()) {
		clipboard += ce.dump() + "\n";
		ce.delete();
		elmList.removeElementAt(i);
	    }
	}
	enablePaste();
	releaseCustomizer();
	needAnalyze();
    }

    void doDelete() {
	int i;
	pushUndo();
	setMenuSelection();
	for (i = elmList.size()-1; i >= 0; i--) {
	    CircuitElm ce = getElm(i);
	    if (ce.isSelected()) {
		ce.delete();
		elmList.removeElementAt(i);
	    }
	}
	releaseCustomizer();
	needAnalyze();
    }

    void doCopy() {
	int i;
	clipboard = "";
	setMenuSelection();
	for (i = elmList.size()-1; i >= 0; i--) {
	    CircuitElm ce = getElm(i);
	    if (ce.isSelected())
		clipboard += ce.dump() + "\n";
	}
	enablePaste();
    }

    void enablePaste() {
	pasteItem.setEnabled(clipboard.length() > 0);
    }
    
    void doPaste() {
	pushUndo();
	clearSelection();
	int i;
	Rectangle oldbb = null;
	for (i = 0; i != elmList.size(); i++) {
	    CircuitElm ce = getElm(i);
	    Rectangle bb = ce.getBoundingBox();
	    if (oldbb != null)
		oldbb = oldbb.union(bb);
	    else
		oldbb = bb;
	}
	int oldsz = elmList.size();
	readSetup(clipboard, true);
	
	// select new items
	Rectangle newbb = null;
	for (i = oldsz; i != elmList.size(); i++) {
	    CircuitElm ce = getElm(i);
	    ce.setSelected(true);
	    Rectangle bb = ce.getBoundingBox();
	    if (newbb != null)
		newbb = newbb.union(bb);
	    else
		newbb = bb;
	}
	System.out.println(oldbb + " " + newbb);
	if (oldbb != null && newbb != null && oldbb.intersects(newbb)) {
	    // find a place for new items
	    int dx = 0, dy = 0;
	    int spacew = circuitArea.width - oldbb.width - newbb.width;
	    int spaceh = circuitArea.height - oldbb.height - newbb.height;
	    if (spacew > spaceh)
		dx = snapGrid(oldbb.x + oldbb.width  - newbb.x + gridSize);
	    else
		dy = snapGrid(oldbb.y + oldbb.height - newbb.y + gridSize);
	    for (i = oldsz; i != elmList.size(); i++) {
		CircuitElm ce = getElm(i);
		ce.move(dx, dy);
	    }
	    // center circuit
	    handleResize();
	}
	needAnalyze();
    }

    void clearSelection() {
	int i;
	for (i = 0; i != elmList.size(); i++) {
	    CircuitElm ce = getElm(i);
	    ce.setSelected(false);
	}
    }
    
    void doSelectAll() {
	int i;
	for (i = 0; i != elmList.size(); i++) {
	    CircuitElm ce = getElm(i);
	    ce.setSelected(true);
	}
    }

    public void keyPressed(KeyEvent e) {}
    public void keyReleased(KeyEvent e) {}
    
    public void keyTyped(KeyEvent e) {
	if (e.getKeyChar() > ' ' && e.getKeyChar() < 127) {
	    Class c = dumpTypes[e.getKeyChar()];
	    if (c == null || c == Scope.class)
		return;
	    CircuitElm elm = null;
	    elm = constructElement(c, 0, 0);
	    if (elm == null || !(elm.needsShortcut() && elm.getDumpClass() == c))
		return;
	    mouseMode = MODE_ADD_ELM;
	    mouseModeStr = c.getName();
	    addingClass = c;
	}
	if (e.getKeyChar() == ' ') {
	    mouseMode = MODE_SELECT;
	    mouseModeStr = "Select";
	}
	tempMouseMode = mouseMode;
    }

    double linearToFrequency(double x) {
	if (linearCheckItem.getState())
	    return x*(maxFrequency-minFrequency) + minFrequency;
	return minFrequency * Math.exp(freqLogRange*x);
    }
    
    double frequencyToLinear(double x) {
	if (linearCheckItem.getState())
	    return (x-minFrequency)/(maxFrequency-minFrequency);
	return (Math.log(x) - minLogFrequency)/freqLogRange;
    }
    
    // factors a matrix into upper and lower triangular matrices by
    // gaussian elimination.  On entry, a[0..n-1][0..n-1] is the
    // matrix to be factored.  ipvt[] returns an integer vector of pivot
    // indices, used in the lu_solve() routine.
    boolean lu_factor(Complex a[][], int n, int ipvt[]) {
	double scaleFactors[];
	int i,j,k;

	scaleFactors = new double[n];
	
        // divide each row by its largest element, keeping track of the
	// scaling factors
	for (i = 0; i != n; i++) { 
	    double largest = 0;
	    for (j = 0; j != n; j++) {
		double x = a[i][j].mag;
		if (x > largest)
		    largest = x;
	    }
	    // if all zeros, it's a singular matrix
	    if (largest == 0)
		return false;
	    scaleFactors[i] = 1.0/largest;
	}

	Complex q = new Complex();
	Complex q1 = new Complex();
	
        // use Crout's method; loop through the columns
	for (j = 0; j != n; j++) {
	    
	    // calculate upper triangular elements for this column
	    for (i = 0; i != j; i++) {
		q.set(a[i][j]);
		for (k = 0; k != i; k++) {
		    q1.set(a[i][k]);
		    q1.mult(a[k][j]);
		    q.subtract(q1);
		}
		a[i][j].set(q);
	    }

	    // calculate lower triangular elements for this column
	    double largest = 0;
	    int largestRow = -1;
	    for (i = j; i != n; i++) {
		q.set(a[i][j]);
		for (k = 0; k != j; k++) {
		    q1.set(a[i][k]);
		    q1.mult(a[k][j]);
		    q.subtract(q1);
		}
		a[i][j].set(q);
		if (q.mag >= largest) {
		    largest = q.mag;
		    largestRow = i;
		}
	    }
	    
	    // pivoting
	    if (j != largestRow) {
		Complex x;
		for (k = 0; k != n; k++) {
		    x = a[largestRow][k];
		    a[largestRow][k] = a[j][k];
		    a[j][k] = x;
		}
		scaleFactors[largestRow] = scaleFactors[j];
	    }

	    // keep track of row interchanges
	    ipvt[j] = largestRow;

	    // avoid zeros
	    if (a[j][j].mag == 0.0) {
		System.out.println("avoided zero");
		a[j][j].set(1e-18);
	    }

	    if (j != n-1) {
		q1.set(a[j][j]);
		q1.recip();
		for (i = j+1; i != n; i++)
		    a[i][j].mult(q1);
	    }
	}
	return true;
    }

    // Solves the set of n linear equations using a LU factorization
    // previously performed by lu_factor.  On input, b[0..n-1] is the right
    // hand side of the equations, and on output, contains the solution.
    void lu_solve(Complex a[][], int n, int ipvt[], Complex b[]) {
	int i;

	// find first nonzero b element
	for (i = 0; i != n; i++) {
	    int row = ipvt[i];

	    Complex swap = b[row];
	    b[row] = b[i];
	    b[i] = swap;
	    if (swap.mag != 0)
		break;
	}
	
	int bi = i++;
	Complex q = new Complex();
	Complex q1 = new Complex();
	for (; i < n; i++) {
	    int row = ipvt[i];
	    int j;
	    q.set(b[row]);
	    
	    b[row].set(b[i]);
	    // forward substitution using the lower triangular matrix
	    for (j = bi; j < i; j++) {
		q1.set(a[i][j]);
		q1.mult(b[j]);
		q.subtract(q1);
	    }
	    b[i].set(q);
	}
	for (i = n-1; i >= 0; i--) {
	    q.set(b[i]);
	    // back-substitution using the upper triangular matrix
	    int j;
	    for (j = i+1; j != n; j++) {
		q1.set(a[i][j]);
		q1.mult(b[j]);
		q.subtract(q1);
	    }
	    q.divide(a[i][i]);
	    b[i].set(q);
	}
    }

}
